/*
 * Hey this is the default GTK+2 control plugin for LAMIP
 * read this for a simple example to build a control plugin.
 * I know this is not really simple as we use toolkit that are not
 * really "clean", but so... we are not perfect as this sofware
 * that need your help to be better than others !!!! :)
 *
 *
 * Frederic Fondriest (c)2003-2005
*/
#include <plug_control.h>
#include <gtk/gtk.h>

#include "libgtk2.h"
#include "main_win.h"
#include "playlist_win.h"
#include "plugin_win.h"
#include "about_win.h"
#ifdef WITH_SHOUTCAST
#include "stream_win.h"
#endif

static int	gtk2_init(const lamipPluginHandle*);
static void	gtk2_run(void);
static void	gtk2_unload(void);
static void	gtk2_on_signal(const struct _lamipEvent*);

static ControlPlugin	control;
gchar*			path_selection = 0;

/*****************************************************************************/
ControlPlugin*	lamip_control_info(void)
{
  control.common.name		= "default";
  control.common.description	= "GTK+-2.x default Control - Frederic Fondriest for LAMIP-0.0.3";
  control.common.init		= gtk2_init;
  control.common.cleanup	= gtk2_unload;
  control.run			= gtk2_run;
  control.signal		= gtk2_on_signal;

  return(&control);
}



static int	gtk2_init(const lamipPluginHandle* conf)
{
  DEBUG("GTK2: initializing...");
  return(1);
}

static void	gtk2_run(void)
{
  playlist_selected	= NULL;

  DEBUG("GTK2: run !\n");
  lamip_setFlagFollow(0);
  gtk_set_locale();
  if(!g_thread_supported()){
    DEBUG("GTK2: thread init");
    g_thread_init(0);
    gdk_threads_init();
    if(!gtk_init_check(NULL,NULL)){
      DEBUG("GTK2: failed to initialize GTK+");
      return;
    }
  }
  DEBUG("GTK2: initialized");
  GDK_THREADS_ENTER();
  if(mainwin_create()==FALSE){
    g_printerr("GTK2: failed to create the main window\n");
    return;
  }
  windowplaylist_create();
  plugin_win_create();
  about_win_create();
#ifdef WITH_SHOUTCAST
  windowstream_create();
#endif
  GDK_THREADS_LEAVE();
  gdk_threads_enter();
  gtk_main();
  gdk_threads_leave();
}

static void	gtk2_unload(void)
{
  DEBUG("GTK2: unload");
}

static void	gtk2_on_signal(const struct _lamipEvent* ev)
{
  if(!ev){
    g_printerr("GTK2: ERROR, bad event received !!!\n");
    return;
  }
  switch(ev->type){
  case EVENT_SONG_CHANGED:
    {
    GtkTreeSelection*	selection	= NULL;
    lamipSonginfo*	songinfo	= NULL;
    gchar*		title		= NULL;
    gchar*		t		= NULL;
    gchar*		song_number	= NULL;
    lamipPlaylist*	playlist	= ev->data.song_changed.pl;
    gint		trk		= ev->data.song_changed.pos;
    gint		sub		= ev->data.song_changed.trk;

    //g_print("GTK2: EVENT_SONG_CHANGED");
    if(!playlist || trk<0 || !sub<0){
#ifdef DEBUGING
      g_printerr("GTK2: bad playlist selection (%s)\n", __FILE__);
#endif
      return;
    }
    GDK_THREADS_ENTER();
    if(gtk_tree_view_get_model(GTK_TREE_VIEW(filename_treeview))==NULL){
      g_printerr("GTK2: %s @ %d Error !!!\n", __FILE__, __LINE__);
      return;
    }
    selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(filename_treeview));
    if(sub)
      song_number = g_strdup_printf("%d:%d", trk, sub);
    else
      song_number = g_strdup_printf("%d", trk);
    if(song_number){
      gtk_tree_selection_select_path(selection,
				     gtk_tree_path_new_from_string(song_number));
      songinfo = lamip_playlist_getSong(playlist, trk);
      if(songinfo==0){
	g_printerr("GTK2: can't get songinfo\n");
      }else{
	t = lamip_songinfo_getTitle(songinfo);
	if(t && !g_utf8_validate(t, -1, NULL)){
	  title = g_locale_to_utf8(t, -1, NULL, NULL, NULL);
	  g_free(t);
	}else{
	  title = t;
	  if(!title){
	    title = lamip_songinfo_getFilename(songinfo);
	    if(title){
	      t = g_filename_to_utf8(title, -1, NULL, NULL, NULL);
	      if(t){
		title = g_path_get_basename(t);
		g_free(t);
	      }
	    }
	  }
	}
      }
    }
    if(!title)
      title = g_strdup("no title");
#ifdef DEBUGING
    g_printerr("Set label to : %s for file  %s\n", title, song_number);
#endif
    gtk_label_set_text(GTK_LABEL(mainwin_songlabel), title);
    g_free(title);
    g_free(song_number);
    gdk_flush();
    GDK_THREADS_LEAVE();
    break;
    }

  case EVENT_SONG_BEGIN:
#ifdef DEBUGING
    g_printerr("GTK2: EVENT_SONG_BEGIN\n");
#endif
    break;

  case EVENT_SONG_END:
    {
#ifdef DEBUGING
      g_printerr("GTK2: EVENT_SONG_END\n");
#endif
      GDK_THREADS_ENTER();
      gtk_widget_set_sensitive(GTK_WIDGET(seekBar), FALSE);
      GDK_THREADS_LEAVE();
      break;
    }

  case EVENT_POS_CHANGED:
    {
      //g_print("GTK2: EVENT_POS_CHANGED\n");
      if(seekBar_pressed==FALSE && lamip_getSongLen()!=0){
	guint	pos = ev->data.pos_changed*255/lamip_getSongLen();

	GDK_THREADS_ENTER();
	gtk_widget_set_sensitive(GTK_WIDGET(seekBar), TRUE);
	gtk_range_set_value(GTK_RANGE(seekBar), pos);
	gdk_flush();
	GDK_THREADS_LEAVE();
      }else if(lamip_getSongLen()==0){
	GDK_THREADS_ENTER();
	gtk_widget_set_sensitive(GTK_WIDGET(seekBar), FALSE);
	GDK_THREADS_LEAVE();
      }
      break;
    }

  case EVENT_PAUSED:
    {
      //g_print("GTK2: EVENT_PAUSED\n");
      GDK_THREADS_ENTER();
      gtk_widget_set_sensitive(GTK_WIDGET(seekBar), FALSE);
      GDK_THREADS_LEAVE();
      break;
    }

  case EVENT_UNPAUSED:
    //g_print("GTK2: EVENT_UNPAUSED\n");
    break;

  case EVENT_META_CHANGED:
    {
      char*	key   = ev->data.meta_changed.key;

      if(!key)
	break;
#ifdef DEBUGING
      g_printerr("GTK2: EVENT_META_CHANGED (%s)\n", key);
#endif
      if(!g_strncasecmp(key, "TITLE", 5)){
	char*	title = ev->data.meta_changed.val;

	GDK_THREADS_ENTER();
	if(title) gtk_label_set_text(GTK_LABEL(mainwin_songlabel), title);
	gtk_widget_queue_draw(GTK_WIDGET(mainwin_songlabel));
	gdk_flush();
	GDK_THREADS_LEAVE();
      }
      break;
    }
  case EVENT_MESSAGE:
    {
#ifdef DEBUGING
      g_printerr("GTK2: MESSAGE - (%s)\n", ev->data.message);
#endif
      break;
    }
 case EVENT_VOLUME_CHANGED:
   {
     break;
   }
  case EVENT_EXIT:
    {
      gtk_main_quit();
      break;
    }
  default:
    g_printerr("GTK2: Unknow Event (you should upgrade this plugin)\n");
    break;
  }

}
