/*
** this is the main gui functions
** to control LAMIP via gtk+-2.x API
**
*/
#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h> // to have list of keybord keys

#include "lamip.h"
#include "libgtk2.h"
#include "plug_control.h"
#include "main_win.h"
#include "playlist_win.h"
#include "plugin_win.h"
#include "about_win.h"
#ifdef WITH_SHOUTCAST
 #include "stream_win.h"
#endif

GtkWidget*	mainwin_songlabel	= 0;
GtkWidget*	mainwin			= 0;
GtkWidget*	seekBar			= 0;
gboolean	seekBar_pressed		= FALSE;
guint		seekBar_pos		= 0;
GtkWidget*	volumeBar		= 0;

static void	cb_mainwin_quit(GtkWidget*, gint);
static gboolean	cb_mainwin_keypress(GtkWidget*, GdkEventKey*, gpointer);
static void	cb_mainwin_open(GtkWidget*, gint);
void		cb_mainwin_openmenu(GtkWidget*, gint);
static void	cb_mainwin_openurl(GtkWidget*, gint);
static void	cb_mainwin_play(GtkWidget*, gint);
static void	cb_mainwin_stop(GtkWidget*, gint);
static void	cb_mainwin_prev(GtkWidget*, gint);
static void	cb_mainwin_next(GtkWidget*, gint);
static void	cb_mainwin_replayfile(GtkWidget*, gint);
static void	cb_mainwin_gapless(GtkWidget*, gint);
static void	cb_mainwin_replayplaylist(GtkWidget*, gint);
static gboolean	cb_mainwin_seekBar_release(GtkWidget*,
					   GdkEventButton*,
					   gpointer);
static gboolean	cb_mainwin_seekBar_press(GtkWidget*,
					 GdkEventButton*,
					 gpointer);


/*
 * Defines all actions for the menubar
*/

static GtkActionEntry mainwin_actionentries[] =
  {
    { "LAMIP", NULL, "LAMIP"},
    { "Options", NULL, "Options"},
    { "View", NULL, "View"},
    { "Playlist", GTK_STOCK_INDEX, "Playlist", NULL,
      "Open the playlist window", G_CALLBACK(cb_windowplaylist_show) },
    { "Plugins options", GTK_STOCK_PREFERENCES, "Plugins options", NULL,
      "Open the plugins options window", G_CALLBACK(cb_windowplugin_show) },
#ifdef WITH_SHOUTCAST
    { "ShoutcastList", GTK_STOCK_NETWORK, "Streams list", NULL,
      "Open the shoutcast window list", G_CALLBACK(cb_windowstream_show) },
#endif
    { "About", GTK_STOCK_ABOUT, "About", NULL,
      "Open the About window", G_CALLBACK(cb_about_win_show) },
    { "Quit", GTK_STOCK_QUIT, "Quit", NULL,
      "Quit lamip control", G_CALLBACK(cb_mainwin_quit) }
  };
static guint mainwin_n_actionentries = G_N_ELEMENTS(mainwin_actionentries);

static GtkToggleActionEntry mainwin_toggleactionentries[] =
  {
    { "ReplayCurrentFile", NULL, "Replay current file", NULL,
      "replay the current selected file",
      G_CALLBACK(cb_mainwin_replayfile),
      FALSE },
    { "ReplayAllFiles", NULL, "Replay all files", NULL,
      "replay all files in the current playlist",
      G_CALLBACK(cb_mainwin_replayplaylist),
      FALSE },
    { "Gapless" , NULL, "Gapless", NULL,
      "don't close/open soundcard betwwen songs",
      G_CALLBACK(cb_mainwin_gapless),
      FALSE }
  };
static guint mainwin_n_toggleactionentries = G_N_ELEMENTS(mainwin_toggleactionentries);


static gchar*	mainwin_menu_entries =
"<ui>"
" <menubar name='MenuBar'>"
"  <menu action='LAMIP'>"
"   <menuitem action='About'/>"
"   <menuitem action='Quit'/>"
"  </menu>"
"  <menu action='View'>"
"   <menuitem action='Playlist'/>"
#ifdef WITH_SHOUTCAST
"   <menuitem action='ShoutcastList'/>"
#endif
"  </menu>"
"  <menu action='Options'>"
"   <menuitem action='ReplayCurrentFile'/>"
"   <menuitem action='ReplayAllFiles'/>"
"   <menuitem action='Gapless'/>"
"   <separator/>"
"   <menuitem action='Plugins options'/>"
"  </menu>"
" </menubar>"
"</ui>";


/*
 * build the mainwindow and calbacks such as play/stop/pause..
*/
gboolean mainwin_create(void)
{
  GtkUIManager*		ui		= NULL;
  GtkActionGroup*	actions		= NULL;
  GtkWidget*		box1		= NULL;
  GtkWidget*		box2		= NULL;
  GtkWidget*		button		= NULL;
  GtkWidget*		icon		= NULL;
  GtkTooltips*		tooltip		= NULL;
  GdkPixbuf*		lamip_icon	= NULL;
  GError*		error		= NULL;

  lamip_icon = gdk_pixbuf_new_from_file(ICON_INSTALL_DIR, &error);
  mainwin = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  g_signal_connect(G_OBJECT(mainwin),
		   "destroy",
		   G_CALLBACK(cb_mainwin_quit),
		   &mainwin);
  gtk_window_set_title(GTK_WINDOW(mainwin),"LAMIP gtk+2 control");
  gtk_window_set_policy(GTK_WINDOW(mainwin), FALSE, FALSE, TRUE);
  gtk_window_set_decorated(GTK_WINDOW(mainwin), TRUE);
  gtk_window_set_icon(GTK_WINDOW(mainwin), lamip_icon);
  gtk_window_set_default_icon(lamip_icon);

  g_signal_connect(mainwin, "key_press_event",
		   G_CALLBACK(cb_mainwin_keypress), 0);

  box1 = gtk_vbox_new (FALSE, 0);
  gtk_container_add(GTK_CONTAINER (mainwin), box1);

  /* create menu */
  actions = gtk_action_group_new("Mainwin Actions");
  gtk_action_group_add_actions(actions, mainwin_actionentries,
			       mainwin_n_actionentries, NULL);
  gtk_action_group_add_toggle_actions(actions, mainwin_toggleactionentries,
				      mainwin_n_toggleactionentries, NULL);
  ui = gtk_ui_manager_new();
  gtk_ui_manager_insert_action_group(ui, actions, 0);
  if(!gtk_ui_manager_add_ui_from_string(ui, mainwin_menu_entries,-1, NULL)){
    g_printerr("building menus failed\n");
    return(FALSE);
  }
  gtk_box_pack_start(GTK_BOX(box1),
		     gtk_ui_manager_get_widget(ui, "/MenuBar"),
		     FALSE, FALSE, 0);

  box2 = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start(GTK_BOX(box1), box2, TRUE, TRUE, 0);


  /* create the song label */
  mainwin_songlabel=gtk_label_new(LAMIP_VERSION);
  gtk_box_pack_start(GTK_BOX(box2),
		     GTK_WIDGET(mainwin_songlabel),
		     FALSE, FALSE, 0);
  gtk_widget_set_size_request(mainwin_songlabel, 200, 20);

  /* create the seeking bar */
  seekBar_pressed = FALSE;
  seekBar_pos = 0;
  seekBar=gtk_hscale_new_with_range(0,255,1);
  gtk_scale_set_draw_value(GTK_SCALE(seekBar), FALSE);
  //gtk_range_set_update_policy(GTK_RANGE(seekBar), GTK_UPDATE_DISCONTINUOUS);
  gtk_widget_set_sensitive(GTK_WIDGET(seekBar), FALSE);  
  g_signal_connect(G_OBJECT(seekBar),
		   "button-press-event",
		   G_CALLBACK(cb_mainwin_seekBar_press),
		   &seekBar);
  
  g_signal_connect(G_OBJECT(seekBar),
		   "button-release-event",
		   G_CALLBACK(cb_mainwin_seekBar_release),
		   &seekBar);
  gtk_box_pack_start(GTK_BOX(box1),
		     seekBar,
		     FALSE, FALSE, 0);

  gtk_widget_set_size_request(seekBar, 100, 15);

  /* create buttons bar */
  box2 = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_end(GTK_BOX(box1), box2, TRUE, TRUE, 0);

  button= gtk_button_new ();
  icon = gtk_image_new_from_stock(GTK_STOCK_MEDIA_PREVIOUS, 
				  GTK_ICON_SIZE_SMALL_TOOLBAR);
  gtk_container_add(GTK_CONTAINER(button), icon);
  gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
  g_signal_connect (button, "clicked",
		    G_CALLBACK(cb_mainwin_prev), NULL);
  tooltip = gtk_tooltips_new();
  gtk_tooltips_set_tip(tooltip, button, "Previous track", "");
  gtk_box_pack_start(GTK_BOX (box2), button, TRUE, TRUE, 0);

  button= gtk_button_new ();
  icon = gtk_image_new_from_stock(GTK_STOCK_MEDIA_STOP, 
				  GTK_ICON_SIZE_SMALL_TOOLBAR);
  gtk_container_add(GTK_CONTAINER(button), icon);
  gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
  tooltip = gtk_tooltips_new();
  gtk_tooltips_set_tip(tooltip, button, "Stop playback", "");
  g_signal_connect (button, "clicked",
		    G_CALLBACK(cb_mainwin_stop), NULL);
  gtk_box_pack_start(GTK_BOX (box2), button, TRUE, TRUE, 0);

  button= gtk_button_new ();
  icon = gtk_image_new_from_stock(GTK_STOCK_MEDIA_PLAY, 
				  GTK_ICON_SIZE_SMALL_TOOLBAR);
  gtk_container_add(GTK_CONTAINER(button), icon);
  gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
  tooltip = gtk_tooltips_new();
  gtk_tooltips_set_tip(tooltip, button, "Play track", "");
  g_signal_connect (button, "clicked",
		    G_CALLBACK(cb_mainwin_play), NULL);
  gtk_box_pack_start(GTK_BOX (box2), button, TRUE, TRUE, 0);

  button= gtk_button_new ();
  icon = gtk_image_new_from_stock(GTK_STOCK_MEDIA_NEXT, 
				  GTK_ICON_SIZE_SMALL_TOOLBAR);
  gtk_container_add(GTK_CONTAINER(button), icon);
  gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
  tooltip = gtk_tooltips_new();
  gtk_tooltips_set_tip(tooltip, button, "Next track", "");
  g_signal_connect (button, "clicked",
		    G_CALLBACK(cb_mainwin_next), NULL);
  gtk_box_pack_start(GTK_BOX (box2), button, TRUE, TRUE, 0);
 
  button= gtk_button_new ();
  icon = gtk_image_new_from_stock(GTK_STOCK_OPEN, 
				  GTK_ICON_SIZE_SMALL_TOOLBAR);
  gtk_container_add(GTK_CONTAINER(button), icon);
  gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
  tooltip = gtk_tooltips_new();
  gtk_tooltips_set_tip(tooltip, button, "Open files", "");
  g_signal_connect (button, "clicked",
		    G_CALLBACK(cb_mainwin_openmenu), NULL);
  gtk_box_pack_start(GTK_BOX (box2), button, TRUE, TRUE, 0);

  /* set root for filechooser */
  if(path_selection) g_free(path_selection);
  path_selection = g_strdup("/");

  gtk_widget_show_all(mainwin);
  return(TRUE);
}

static void	mainwin_clean(void)
{
  if(path_selection)
    g_free(path_selection);
  if(mainwin)
    gtk_widget_destroy(mainwin);
  mainwin = 0;
}

static void	cb_mainwin_quit(GtkWidget *w, gint val)
{
  windowplaylist_clean();
  plugin_win_clean();
  mainwin_clean();
  gtk_main_quit();
}

static void	cb_mainwin_play(GtkWidget *w, gint val)
{
  lamip_play();
}

static void	cb_mainwin_stop(GtkWidget *w, gint val)
{
  gtk_widget_set_sensitive(GTK_WIDGET(seekBar), FALSE);
  lamip_stop();
}

static void	cb_mainwin_prev(GtkWidget *w, gint val)
{
  lamip_prev();
}

void		cb_mainwin_next(GtkWidget *w, gint val)
{
  lamip_next();
}

/*****************************************************************************/
/*
 * the 'open' create a new clean playlist and add it to the list of playlists
 *
*/

void		cb_mainwin_openmenu(GtkWidget* w, gint val)
{
  GtkWidget*	menu;
  GtkWidget*	item;
  GtkWidget*	icon;
  

  menu = gtk_menu_new();
  item = gtk_image_menu_item_new_with_label("Open files");
  icon = gtk_image_new_from_stock(GTK_STOCK_FILE, 
				  GTK_ICON_SIZE_SMALL_TOOLBAR);
  gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item), icon);
  gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
  g_signal_connect(G_OBJECT(item),
		   "activate",
		   G_CALLBACK(cb_mainwin_open),
		   0);
  gtk_widget_show(item);

  item = gtk_image_menu_item_new_with_label("Open directories");
  icon = gtk_image_new_from_stock(GTK_STOCK_DIRECTORY, 
				  GTK_ICON_SIZE_SMALL_TOOLBAR);
  gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item), icon);
  gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
  g_signal_connect(G_OBJECT(item),
		   "activate",
		   G_CALLBACK(cb_mainwin_open),
		   (void*)1);
  gtk_widget_show(item);

  item = gtk_image_menu_item_new_with_label("Open URL");
  icon = gtk_image_new_from_stock(GTK_STOCK_NETWORK, 
				  GTK_ICON_SIZE_SMALL_TOOLBAR);
  gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item), icon);
  gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
  g_signal_connect(G_OBJECT(item),
		   "activate",
		   G_CALLBACK(cb_mainwin_openurl),
		   0);
  gtk_widget_show(item);

  gtk_menu_popup(GTK_MENU(menu), 0, 0, 0, 0, 1,
		 gtk_get_current_event_time());
}

void		cb_mainwin_open(GtkWidget* w, gint val)
{
  GtkTreeSelection* selection;

  DEBUG("cb_mainwin_open");
  add_new_playlist(val);
  /* set to first song selected for the playlist window */
  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(playlist_treeview));
  gtk_tree_selection_select_path(selection,
				 gtk_tree_path_new_from_string("0"));
  //playlist_selected = playlist;
}


static GtkWidget*	entry  = NULL;

static void		cb_mainwin_gourl(GtkWidget* w, gint val)
{
  lamipSonginfo*	info;
  lamipPlaylist*	playlist;

#ifdef DEBUGING
  g_printerr("GTK2: go to url (%s)\n", gtk_entry_get_text(GTK_ENTRY(entry)));
#endif
  lamip_stop();
  playlist_clean();
  playlist = lamip_playlist_new();
  //lamip_playlist_set(playlist, 0, 0);
  playlist_add_item(playlist); // add the list to the list of playlist
  
  if(!(info = lamip_songinfo_new())){
    return;
  }
  if(!lamip_songinfo_read(gtk_entry_get_text(GTK_ENTRY(entry)), info)){
    lamip_songinfo_destroy(info);
    return;
  }
  lamip_playlist_append(playlist, info);
  lamip_play();
}

static void		cb_mainwin_openurl(GtkWidget* w, gint val)
{
  static GtkWidget*	winurl = NULL;
  GtkWidget*		hbox;
  GtkWidget*		vbox;
  GtkWidget*		label;
  GtkWidget*		button;

  if(winurl)
    return;
  winurl = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title(GTK_WINDOW(winurl), "Enter a full url");
  gtk_window_set_resizable(GTK_WINDOW(winurl), FALSE);
  g_signal_connect (winurl, "destroy",
		    G_CALLBACK(gtk_widget_destroyed), &winurl);

  vbox = gtk_vbox_new(FALSE, 5);
  gtk_container_add(GTK_CONTAINER(winurl), vbox);
  hbox = gtk_hbox_new(FALSE, 5);
  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);

  label = gtk_label_new(NULL);
  gtk_label_set_text(GTK_LABEL(label), "URL :");
  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 5);
  entry = gtk_entry_new();
  gtk_box_pack_start(GTK_BOX(hbox), entry, FALSE, FALSE, 5);
  button = gtk_button_new();
  gtk_button_set_label(GTK_BUTTON(button), "Open URL !");
  gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 0);
  g_signal_connect(G_OBJECT(button),
		   "clicked",
		   G_CALLBACK(cb_mainwin_gourl),
		   0);  
  gtk_widget_show_all(winurl);
}
/*****************************************************************************/

static void	cb_mainwin_replayfile(GtkWidget *w, gint val)
{
  static gboolean flag = FALSE;

  if(flag == TRUE){
    flag=FALSE;
    lamip_setOrder(ORDER_ONE_BY_ONE);
  }else{
    flag=TRUE;
    lamip_setOrder(ORDER_REPEAT_FILE);
  }

}

static void	cb_mainwin_replayplaylist(GtkWidget *w, gint val)
{
  static gboolean flag=FALSE;

  if(flag == TRUE){
    flag=FALSE;
    lamip_setOrder(ORDER_ONE_BY_ONE);
  }else{
    flag=TRUE;
    lamip_setOrder(ORDER_REPEAT_PLAYLIST);
  }
}

static void	cb_mainwin_gapless(GtkWidget* w, gint val)
{
  static gboolean flag = FALSE;

  if(flag==TRUE){
    flag = FALSE;
    lamip_setGapless(0);
  }else{
    flag = TRUE;
    lamip_setGapless(1);
  }
}


static gboolean	cb_mainwin_seekBar_release(GtkWidget *widget,
					   GdkEventButton *event,
					   gpointer user_data)
{
#ifdef DEBUG
  g_printerr("GTK2: SEEKING TO - %d\n", (gint)gtk_range_get_value(GTK_RANGE(widget)));
#endif
  seekBar_pressed = FALSE;
  seekBar_pos = gtk_range_get_value(GTK_RANGE(widget));
  if(lamip_getSongLen()!=0){
    g_printerr("GTK2: %d\n", (guint)(lamip_getSongLen()*(seekBar_pos/255.)));
    lamip_seek((guint)(lamip_getSongLen()*(seekBar_pos/255.)) );
  }
  return(FALSE);
}

static gboolean	cb_mainwin_seekBar_press(GtkWidget *widget,
					 GdkEventButton *event,
					 gpointer user_data)
{
#ifdef DEBUG
  g_printerr("SEEK: pressed\n");
#endif
  seekBar_pressed = TRUE;
  return(FALSE);
}

/*
 * when a keyboard key is pressed
 * do some actions..
 *
 * note from ciberfred.
 * as we are on Xserver we could use keybinding from it no ?
 * well maybe it could help some people to have this here.
 *
*/
static gboolean cb_mainwin_keypress(GtkWidget* grab_widget, GdkEventKey* event, gpointer data)
{
#ifdef DEBUG
  g_printerr("GTK2: key pressed\n");
#endif
  switch (event->keyval){
  case GDK_KP_Add:
    {
      //lamip_set_Volume(++);
      break;
    }
  case GDK_KP_Subtract:
    {
      //lamip_setVolume(--);
      break;
    }
  case GDK_Left:
    {
      lamip_prev();
      break;
    }
  case GDK_Right:
    {
      lamip_next();
      break;
    }
  case GDK_C:
  case GDK_c:
    {
      lamip_play();
      break;
    }
  case GDK_l:
  case GDK_L:
    {
      if(event->state & GDK_CONTROL_MASK){	/* CTRL+L */
	cb_mainwin_openurl(0,0);
      }else if(event->state & GDK_SHIFT_MASK){	/* SHIFT+L */
	cb_mainwin_open(0, 1);
      }else{
	cb_mainwin_open(0, 0);
      }
      break;
    }
  case GDK_P:
  case GDK_p:
    {
      cb_windowplaylist_show(0,0);
      break;
    }
  case GDK_Q:
  case GDK_q:
    {
      cb_mainwin_quit(0, 0);
      break;
    }
#ifdef WITH_SHOUTCAST
  case GDK_S:
  case GDK_s:
    {
      cb_windowstream_show(0,0);
      break;
    }
#endif
  case GDK_X:
  case GDK_x:
    {
      lamip_stop();
      break;
    }
  case GDK_F4:
    {
      if((event->state & GDK_CONTROL_MASK) ||	/* CTRL+F4 */
	 (event->state & GDK_MOD1_MASK))	/* ALT+F4  */
	cb_mainwin_quit(0, 0);
      break;
    }
  default:
    {
      return(FALSE);
    }
  }
  return (TRUE);
}
