/*
 * playlist window management
*/

#include <stdlib.h>
#include <plug_control.h>

#include "libgtk2.h"
#include "playlist_win.h"


/* the store description of the gtk+ playlist */
enum {
  COLUMN_FILENAME,
  COLUMN_SUBTRACK,
  N_COLUMNS
};

/* the store description of the gtk+ list of playlists */
enum {
  PLAYLIST_NAME,
  PLAYLIST_LAMIP_POINTER,
  N_PLAYLIST_COLUMNS
};

#define PLAYLIST_WIN_TITLE	"LAMIP gtk+2 control gui - playlists"


static GtkWidget*       gtkTitle        = 0;
static GtkWidget*       gtkArtist       = 0;
static GtkWidget*       gtkAlbum        = 0;
static GtkWidget*       gtkTrack        = 0;
static GtkWidget*       gtkYear         = 0;
static GtkWidget*       gtkGenre        = 0;
static gboolean		playlist_show	= FALSE;
static gint32		numplaylist	= 0;


static void	_windowplaylist_update_songinfo(lamipPlaylist*, gint, gint);

static GtkTreeModel*	playlist_model;
static GtkTreeModel*	playlist_create_model(void);
void			playlist_clean(void);
static void	cb_playlist_button_add(GtkWidget*, gpointer);
static void	cb_playlist_button_remove(GtkWidget*, gpointer);
/*static void	cb_playlist_button_sort(GtkWidget*, gpointer);*/
static void	cb_playlist_changed(GtkTreeSelection*, gpointer*);

static GtkTreeModel*	filename_model;
static GtkTreeModel*	filename_create_model(void);
static void		filename_clean(void);
static void		filename_add_item(lamipSonginfo*);
static void	cb_filename_button_add(GtkWidget*, gpointer);
static void	cb_filename_button_remove(GtkWidget*, gpointer);
/*static void	cb_filename_button_sort(GtkWidget*, gpointer);*/
static void	cb_filename_changed(GtkTreeSelection*, gpointer*);
static gboolean cb_filename_mouse_press(GtkWidget*, GdkEventButton*, gpointer);

/*****************************************************************************/
/*
 * create the playlist gtkWindow
 */
gboolean        windowplaylist_create(void)
{
  GtkWidget*	sw;
  GtkWidget*	vbox;
  GtkWidget*	vbox2;
  GtkWidget*	hbox;
  GtkWidget*	table;
  GtkWidget*	hpaned;
  GtkWidget*	vpaned;
  GtkWidget*	button;
  GtkWidget*	label;
  GtkWidget*	frame;
  GtkCellRenderer*	cell_renderer;
  GtkTreeViewColumn*	column;


  if(playlist_win)
    return(FALSE);
  playlist_win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title(GTK_WINDOW(playlist_win), PLAYLIST_WIN_TITLE);
  gtk_window_set_decorated(GTK_WINDOW(playlist_win), TRUE);
  /* when we close the window */
  g_signal_connect(G_OBJECT(playlist_win),
		   "delete-event",
		   G_CALLBACK(cb_windowplaylist_show),
		   NULL);
  /* create paned */
  hbox = gtk_hbox_new(TRUE, 0);
  gtk_container_add(GTK_CONTAINER(playlist_win), hbox);
  hpaned = gtk_hpaned_new();
  gtk_box_pack_start(GTK_BOX(hbox), hpaned, TRUE, TRUE, 0);
  vpaned = gtk_vpaned_new();
  gtk_paned_add1(GTK_PANED(hpaned), vpaned);

  /* the list of playlist */
  vbox = gtk_vbox_new(FALSE, 0);
  gtk_paned_add1(GTK_PANED(vpaned), vbox);
  sw = gtk_scrolled_window_new (NULL, NULL);
  gtk_box_pack_start(GTK_BOX(vbox), sw, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
				 GTK_POLICY_NEVER,
				 GTK_POLICY_AUTOMATIC);
  playlist_model = playlist_create_model();
  playlist_treeview = gtk_tree_view_new_with_model(playlist_model);
  g_object_unref(G_OBJECT(playlist_model));
  gtk_container_add(GTK_CONTAINER(sw), playlist_treeview);
  cell_renderer=gtk_cell_renderer_text_new();
  column=gtk_tree_view_column_new_with_attributes("Playlists",
						  cell_renderer,
						  "text",
						  PLAYLIST_NAME,
						  NULL);
  gtk_tree_view_append_column(GTK_TREE_VIEW(playlist_treeview), column);
  {
    GtkTreeSelection* select =
      gtk_tree_view_get_selection(GTK_TREE_VIEW(playlist_treeview));
    gtk_tree_selection_set_mode(select, GTK_SELECTION_SINGLE);
    /* when we click on a playlist of the list */
    g_signal_connect(G_OBJECT(select), "changed",
		     G_CALLBACK(cb_playlist_changed),
		     NULL);

  }
  /* buttons of list of playlists */
  hbox = gtk_hbox_new(TRUE, 4);
  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
  button = gtk_button_new_with_label("Create playlist");
  g_signal_connect(button, "clicked",
		   G_CALLBACK(cb_playlist_button_add),
		   playlist_model);
  gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, TRUE, 0);
  button = gtk_button_new_with_label("Remove playlist");
  g_signal_connect(button, "clicked",
		   G_CALLBACK(cb_playlist_button_remove),
		   playlist_treeview);
  gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, TRUE, 0);
  /*
    button = gtk_button_new_with_label ("Sort playlist");
    g_signal_connect(button, "clicked",
     G_CALLBACK(cb_button_sort_playlist),
     treeview);
    gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, TRUE, 0);
  */

  /* tag informations */
  vbox2 = gtk_vbox_new(FALSE, 0);
  gtk_paned_add2(GTK_PANED(vpaned), vbox2);

  frame = gtk_frame_new("tag informations :");
  gtk_box_pack_start(GTK_BOX(vbox2), frame, TRUE, TRUE, 5);

  table = gtk_table_new(5, 4, FALSE);
  gtk_container_set_border_width(GTK_CONTAINER(table), 5);
  gtk_container_add(GTK_CONTAINER(frame), table);

  label = gtk_label_new("title :");
  gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
  gtk_table_attach(GTK_TABLE(table), label,
		   0, 1, 0, 1,
		   GTK_FILL, GTK_FILL, 0, 0);
  gtkTitle = gtk_entry_new();
  gtk_editable_set_editable(GTK_EDITABLE(gtkTitle), FALSE);
  gtk_table_attach(GTK_TABLE(table), gtkTitle,
		   1, 4, 0, 1,
		   GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		   GTK_FILL|GTK_SHRINK, 0, 0);

  label = gtk_label_new("artist(s) :");
  gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
  gtk_table_attach(GTK_TABLE(table), label,
		   0, 1, 1, 2,
		   GTK_FILL, GTK_FILL, 0, 0);
  gtkArtist = gtk_entry_new();
  gtk_editable_set_editable(GTK_EDITABLE(gtkArtist), FALSE);
  gtk_table_attach(GTK_TABLE(table), gtkArtist,
		   1, 4, 1, 2,
		   GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		   GTK_FILL|GTK_SHRINK, 0, 0);

  label = gtk_label_new("album :");
  gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
  gtk_table_attach(GTK_TABLE(table), label,
		   0, 1, 2, 3,
		   GTK_FILL, GTK_FILL, 0, 0);
  gtkAlbum = gtk_entry_new();
  gtk_editable_set_editable(GTK_EDITABLE(gtkAlbum), FALSE);
  gtk_table_attach(GTK_TABLE(table), gtkAlbum,
		   1, 4, 2, 3,
		   GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		   GTK_FILL|GTK_SHRINK, 0, 0);

  label = gtk_label_new("track :");
  gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
  gtk_table_attach(GTK_TABLE(table), label,
		   0, 1, 3, 4,
		   GTK_FILL, GTK_FILL, 0, 0);
  gtkTrack = gtk_entry_new();
  gtk_editable_set_editable(GTK_EDITABLE(gtkTrack), FALSE);
  gtk_table_attach(GTK_TABLE(table), gtkTrack,
		   1, 2, 3, 4,
		   GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		   GTK_FILL|GTK_SHRINK, 0, 0);

  label = gtk_label_new("year :");
  gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
  gtk_table_attach(GTK_TABLE(table), label,
		   2, 3, 3, 4,
		   GTK_FILL, GTK_FILL, 0, 0);
  gtkYear = gtk_entry_new();
  gtk_editable_set_editable(GTK_EDITABLE(gtkYear), FALSE);
  gtk_table_attach(GTK_TABLE(table), gtkYear,
		   3, 4, 3, 4,
		   GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		   GTK_FILL|GTK_SHRINK, 0, 0);

  label = gtk_label_new("genre :");
  gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
  gtk_table_attach(GTK_TABLE(table), label,
		   0, 1, 4, 5,
		   GTK_FILL|GTK_SHRINK, GTK_FILL, 0, 0);
  gtkGenre = gtk_entry_new();
  gtk_editable_set_editable(GTK_EDITABLE(gtkGenre), FALSE);
  gtk_table_attach(GTK_TABLE(table), gtkGenre,
		   1, 4, 4, 5,
		   GTK_FILL|GTK_EXPAND|GTK_SHRINK,
		   GTK_FILL|GTK_SHRINK, 0, 0);

  /* create the scrolling filename */
  vbox = gtk_vbox_new(FALSE, 8);
  gtk_paned_add2(GTK_PANED(hpaned), vbox);
  sw = gtk_scrolled_window_new (NULL, NULL);
  gtk_box_pack_start(GTK_BOX(vbox), sw, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
				 GTK_POLICY_NEVER,
				 GTK_POLICY_AUTOMATIC);
  /* add the list of songs... */
  filename_model = filename_create_model();
  filename_treeview = gtk_tree_view_new_with_model(filename_model);
  g_object_unref(G_OBJECT(filename_model));
  gtk_container_add(GTK_CONTAINER (sw), filename_treeview);
  cell_renderer=gtk_cell_renderer_text_new();
  column=gtk_tree_view_column_new_with_attributes("Files",
						  cell_renderer,
						  "text",
						  COLUMN_FILENAME,
						  NULL);
  gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_FIXED);
  gtk_tree_view_append_column(GTK_TREE_VIEW(filename_treeview), column);
  {
    GtkTreeSelection* select =
      gtk_tree_view_get_selection(GTK_TREE_VIEW(filename_treeview));
    gtk_tree_selection_set_mode(select, GTK_SELECTION_SINGLE);
    /* to update tag fields */
    g_signal_connect(G_OBJECT(select), "changed",
		     G_CALLBACK(cb_filename_changed),
		     NULL);
    /* to select a new song to play */
    g_signal_connect(GTK_TREE_VIEW(filename_treeview), "button_press_event",
		     G_CALLBACK(cb_filename_mouse_press),
		     GTK_TREE_MODEL(filename_model));
  }

  /* bottom buttons */
  hbox = gtk_hbox_new(TRUE, 4);
  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
  button = gtk_button_new_from_stock(GTK_STOCK_ADD);
  g_signal_connect(button, "clicked",
		   G_CALLBACK(cb_filename_button_add),
		   filename_model);
  gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
  button = gtk_button_new_from_stock(GTK_STOCK_REMOVE);
  g_signal_connect(button, "clicked",
		   G_CALLBACK(cb_filename_button_remove),
		   filename_treeview);
  gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
  /*
    button = gtk_button_new_with_label ("Sort");
    g_signal_connect(button, "clicked",
                     G_CALLBACK(cb_filename_button_sort),
                     filename_treeview);
    gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
  */
  gtk_widget_hide(playlist_win);

  return(TRUE);
}

/*
 * cleanup all gtk things also cleanup all no selected playlists
 */
void    windowplaylist_clean(void)
{
  playlist_clean();
  if(playlist_win) gtk_widget_destroy(playlist_win);
  playlist_win = 0;
}

/*
 * switch on/off view the playlist window (call from mainwindow)
 */
int             cb_windowplaylist_show(GtkWidget* w, gint val)
{
  if(playlist_show==FALSE){
    gtk_widget_show_all(playlist_win);
    playlist_show=TRUE;
  }else{
    gtk_widget_hide(playlist_win);
    playlist_show=FALSE;
  }
  return(TRUE);
}

/*
 * update the tag part of the window
*/

static void	_windowplaylist_update_songinfo(lamipPlaylist* playlist,
						gint	songNumber,
						gint	subtrack)
{
  lamipSonginfo*	songinfo;
  gchar*		value;
  gchar*		string;

  songinfo = lamip_playlist_getSong(playlist, songNumber);

  lamip_songinfo_setActiveTrack(songinfo, subtrack);
  value = lamip_songinfo_getTitle(songinfo);
  if(!value)
    value=g_strdup("");
  string = g_filename_to_utf8(value, -1, NULL, NULL, NULL);
  if(!string)
    string = g_strdup("bad utf8 title");
  gtk_entry_set_text(GTK_ENTRY(gtkTitle), string);
  g_free(value);
  g_free(string);

  value = lamip_songinfo_getArtist(songinfo);
  if(!value)
    value=g_strdup("");
  string = g_filename_to_utf8(value, -1, NULL, NULL, NULL);
  if(!string)
    string = g_strdup("bad utf8 filename");
  gtk_entry_set_text(GTK_ENTRY(gtkArtist), string);
  g_free(value);
  g_free(string);

  value = lamip_songinfo_getAlbum(songinfo);
  if(!value)
    value=g_strdup("");
  string = g_filename_to_utf8(value, -1, NULL, NULL, NULL);
  if(!string)
    string = g_strdup("bad utf8 album");
  gtk_entry_set_text(GTK_ENTRY(gtkAlbum), string);
  g_free(value);
  g_free(string);

  value = lamip_songinfo_getTrack(songinfo);
  if(!value)
    value=g_strdup("");
  string = g_filename_to_utf8(value, -1, NULL, NULL, NULL);
  if(!string)
    string = g_strdup("bad utf8 track");
  gtk_entry_set_text(GTK_ENTRY(gtkTrack), string);
  g_free(value);
  g_free(string);

  value = lamip_songinfo_getYear(songinfo);
  if(!value)
    value=g_strdup("");
  string = g_filename_to_utf8(value, -1, NULL, NULL, NULL);
  if(!string)
    string = g_strdup("bad utf8 year");
  gtk_entry_set_text(GTK_ENTRY(gtkYear), string);
  g_free(value);
  g_free(string);

  value = lamip_songinfo_getType(songinfo);
  if(!value)
    value=g_strdup("");
  string = g_filename_to_utf8(value, -1, NULL, NULL, NULL);
  if(!string)
    string = g_strdup("bad utf8 type");
  gtk_entry_set_text(GTK_ENTRY(gtkGenre), string);
  g_free(value);
  g_free(string);

  /* please don't do that !!! songinfo is CONST !! and managed by lamip */
  //g_free(songinfo);
}

/****************************************************************************/

static GtkTreeModel*	playlist_create_model(void)
{
  GtkListStore*		store;

  store = gtk_list_store_new(N_PLAYLIST_COLUMNS,
			     G_TYPE_STRING,
			     G_TYPE_POINTER);
  return(GTK_TREE_MODEL(store));
}

void		playlist_clean(void)
{
  GtkTreeIter	iter;
  gboolean	valid;

  valid = gtk_tree_model_get_iter_first (playlist_model, &iter);
  while(valid){
    lamipPlaylist*	playlist;
    gchar*		playlist_name;

    gtk_tree_model_get(playlist_model, &iter,
                       PLAYLIST_LAMIP_POINTER, &playlist,
                       PLAYLIST_NAME, &playlist_name,
                       -1);
    lamip_playlist_destroy(playlist);
    g_free(playlist_name);
    valid = gtk_tree_model_iter_next(playlist_model, &iter);
  }
  gtk_list_store_clear(GTK_LIST_STORE(playlist_model));
  numplaylist=0;
  filename_clean();
  playlist_selected = NULL;
}

void		playlist_add_item(lamipPlaylist* playlist)
{
  GtkTreeIter	iter;
  gchar*	string;

  DEBUG("playlist_add_item");
  string = g_strdup_printf("playlist %d", ++numplaylist);
  gtk_list_store_append(GTK_LIST_STORE(playlist_model), &iter);
  gtk_list_store_set(GTK_LIST_STORE(playlist_model), &iter,
                     PLAYLIST_NAME,
                     string,
                     PLAYLIST_LAMIP_POINTER,
                     playlist,
                     -1);
  g_free(string);
}

static void	cb_playlist_button_add(GtkWidget* widget, gpointer data)
{
  DEBUG("cb_playlist_button_add");
  add_new_playlist(0);
}

static void	cb_playlist_button_remove(GtkWidget* widget, gpointer data)
{
  GtkTreeIter		iter;
  GtkTreeView*		treeview = (GtkTreeView *)data;
  GtkTreeModel*		model = gtk_tree_view_get_model(treeview);
  GtkTreeSelection*	selection = gtk_tree_view_get_selection(treeview);

  DEBUG("cb_playlist_button_remove");
  if(gtk_tree_selection_get_selected(selection, NULL, &iter)){
    gint		i;
    GtkTreePath*	path;

    path = gtk_tree_model_get_path(model, &iter);
    i = gtk_tree_path_get_indices(path)[0];
    gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
    filename_clean();
    gtk_tree_path_free(path);

    path = gtk_tree_path_new_from_string("0");
    gtk_tree_selection_select_path(selection, path);
    if(gtk_tree_selection_path_is_selected(selection, path) == TRUE){
      gtk_tree_path_free(path);
    }else{
      lamip_playlist_set(0,0,0);
    }
  }
}

/*
static void	cb_button_sort_playlist(GtkWidget* widget, gpointer data);
{
}
*/

static void	cb_playlist_changed(GtkTreeSelection* selection, gpointer* data)
{
  GtkTreeModel*	model;
  GtkTreeIter	iter;

  DEBUG("cb_playlist_changed");
  if(gtk_tree_selection_get_selected(selection, &model, &iter)){
    gchar*		playlist_name;
    lamipPlaylist*	playlist;
    lamipSonginfo*	cur_songinfo;
    int			i;
    GtkTreePath*	path;

    gtk_tree_model_get(playlist_model, &iter, 
		       PLAYLIST_NAME, &playlist_name,
		       PLAYLIST_LAMIP_POINTER, &playlist,
		       -1);
#ifdef DEBUGING
    g_printerr("GTK2_DEBUG: new playlist selected (%s)\n", playlist_name);
#endif
    playlist_selected = playlist;
    lamip_playlist_set(playlist, 0, 0);
    filename_clean();
    i = 0;
    while((cur_songinfo = lamip_playlist_getSong(playlist, i)) != 0){
      filename_add_item(cur_songinfo);
      i++;
    }
    selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(filename_treeview));
    path = gtk_tree_path_new_from_string("0");
    gtk_tree_selection_select_path(selection, path);
    gtk_tree_path_free(path);
  }else{
    DEBUG("no playlist selected");
  }
}

/*****************************************************************************/

static GtkTreeModel*	filename_create_model(void)
{
  GtkTreeStore*	store;

  store = gtk_tree_store_new(N_COLUMNS,
			     G_TYPE_STRING,
			     G_TYPE_INT);
  return(GTK_TREE_MODEL(store));
}

static void     filename_clean(void)
{
  gtk_tree_store_clear(GTK_TREE_STORE(filename_model));
}

static void		filename_add_item(lamipSonginfo* info)
{
  GtkTreeIter	iter;
  GtkTreeIter	parent;
  gchar*	string;
  gchar*	string2;
  char*		fullpath;
  int		tracks;

  DEBUG("filename_add_item");
  tracks = lamip_songinfo_getTotalTracks(info);
  if(tracks>1)
    fullpath = lamip_songinfo_getTitle(info);
  else
    fullpath = lamip_songinfo_getFilename(info);
  string = g_filename_to_utf8(fullpath, -1, NULL, NULL, NULL);
  if(!string)
    string = g_strdup("bad utf8 filename");
  string2 = g_path_get_basename(string);
  free(fullpath);
  g_free(string);
  gtk_tree_store_append(GTK_TREE_STORE(filename_model), &iter, NULL);
  gtk_tree_store_set(GTK_TREE_STORE(filename_model), &iter,
                     COLUMN_FILENAME, string2,
		     COLUMN_SUBTRACK, -1,
                     -1);
  g_free(string2);
  parent = iter;
  if(tracks>1){
    int trk;

    for(trk=0; trk<tracks; trk++){
      char* title;

      lamip_songinfo_setActiveTrack(info, trk);
      title = lamip_songinfo_getFilename(info);
      string = g_filename_to_utf8(title, -1, NULL, NULL, NULL);
      free(title);
      if(!string)
        string = g_strdup("bad utf8 filename");
      string2 = g_path_get_basename(string);
      g_free(string);
      gtk_tree_store_append(GTK_TREE_STORE(filename_model), &iter, &parent);
      gtk_tree_store_set(GTK_TREE_STORE(filename_model), &iter,
                         COLUMN_FILENAME, string2,/* it's the entry filename */
                         COLUMN_SUBTRACK, trk,
                         -1);
      g_free(string2);
    }
    lamip_songinfo_setActiveTrack(info, 0);
  }
}

static void	cb_filename_button_add(GtkWidget* widget, gpointer data)
{
  GtkTreeSelection* selection;

  DEBUG("cb_filename_button_add");
  add_new_files_to_current_playlist(0);

  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(playlist_treeview));
  gtk_tree_selection_select_path(selection,
				 gtk_tree_path_new_from_string("0"));
  cb_playlist_changed(selection,0);
}

static void	cb_filename_button_remove(GtkWidget* widget, gpointer data)
{
  DEBUG("cb_filename_button_remove");


}

/*
static void	cb_button_sort_playlist(GtkWidget* widget, gpointer data);
{
}
*/


static void	cb_filename_changed(GtkTreeSelection* selection, gpointer* data)
{
  GtkTreeModel*	model;
  GtkTreeIter	iter;

  DEBUG("cb_filename_changed");
  if(gtk_tree_selection_get_selected(selection, &model, &iter)){
    GtkTreePath*	path;
    gint*		indices;
    gchar*		string;
    gint		depth;
    int			track;
    int			subtrack;

    path = gtk_tree_model_get_path(model, &iter);
    depth = gtk_tree_path_get_depth(path);
    indices = gtk_tree_path_get_indices(path);
    string = gtk_tree_path_to_string(path);
#ifdef DEBUGING
    g_printerr("GTK2 : new file selected (%s)\n", string);
#endif
    if(depth == 2) /* a subtrack */
      subtrack = indices[1];
    else
      subtrack = 0;
    track = indices[0];
    lamip_playlist_set(playlist_selected,
		       track,
		       subtrack);
    _windowplaylist_update_songinfo(playlist_selected,
				    track, subtrack);
    g_free(string);
    gtk_tree_path_free(path);
  }
}

static gboolean cb_filename_mouse_press(GtkWidget*	widget,
					GdkEventButton*	event,
					gpointer	callback_data)
{
  DEBUG("cb_filename_mouse_press");
  if(event->button == 1 && event->type == GDK_2BUTTON_PRESS){
    /* double clicked */;
    lamip_play();
    return(FALSE);
  }else if(event->button == 1 && event->type == GDK_BUTTON_PRESS){
    /* clicked */
    return(FALSE);
  }
  return(TRUE);
}

/****************************************************************************/

static void _add_file(char* path)
{
  lamipSonginfo*	info;

  info = lamip_songinfo_new();
  if(!info)
    return;
  if(!lamip_songinfo_read(path, info)){
    lamip_songinfo_destroy(info);
    return;
  }
  lamip_playlist_append(playlist_selected, info);
}

static void _add_files_recursively(char* path, lamipPlaylist* playlist)
{
  GDir*		dir;
  const gchar*	filename;

  filename = 0;
  dir = g_dir_open(path, 0, 0);
  while((filename = g_dir_read_name(dir)) != NULL){
    char* fullpath;

    fullpath = g_strconcat(path, "/", filename, NULL);
    if(g_file_test(fullpath, G_FILE_TEST_IS_DIR)){
      _add_files_recursively((char*)fullpath, playlist);
    }else{
      _add_file(fullpath);
#ifdef DEBUGING
      g_printerr("GTK2: +++ %s\n", fullpath);
#endif
    }
    g_free(fullpath);
  }
  g_dir_close(dir);
}


void	add_new_playlist(gint val)
{
  GtkWidget*		dialog;
  lamipPlaylist*	playlist;

  DEBUG("+ add_new_playlist +");
  dialog = gtk_file_chooser_dialog_new ("Create a new playlist from Files or directories recursively",
                                        GTK_WINDOW(playlist_win),
                                        val ? GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER : GTK_FILE_CHOOSER_ACTION_OPEN,
                                        GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                        GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
                                        NULL);
  gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dialog), path_selection);
  gtk_file_chooser_set_select_multiple(GTK_FILE_CHOOSER(dialog), TRUE);
  if(gtk_dialog_run(GTK_DIALOG (dialog))==GTK_RESPONSE_ACCEPT){
    GSList*	list;
    lamipPlaylist*	playlist_tmp = 0;

    list = gtk_file_chooser_get_filenames(GTK_FILE_CHOOSER (dialog));
    if(list){
      lamip_stop();
      playlist = lamip_playlist_new();
      //lamip_playlist_set(playlist, 0, 0);
      playlist_add_item(playlist);
    }else{
      gtk_widget_destroy (dialog);
      return;
    }
    playlist_tmp = playlist_selected;
    playlist_selected = playlist;
    while(list){
      char*	filename = list->data;

      if(g_file_test(filename, G_FILE_TEST_IS_DIR)){
	_add_files_recursively(filename, playlist);
      }else{
	_add_file(filename);
      }
      list = g_slist_next(list);
    }
    g_slist_free(list);
    playlist_selected = playlist_tmp;
  }
  if(path_selection)
    g_free(path_selection);
  path_selection = gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER(dialog));
  gtk_widget_destroy (dialog);

}

void	add_new_files_to_current_playlist(gint val)
{
  GtkWidget*	dialog;

  DEBUG("+ add_file_to_current_playlist +");
  dialog = gtk_file_chooser_dialog_new("Add Files or directories recursively",
				       GTK_WINDOW(playlist_win),
				       val ? GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER : GTK_FILE_CHOOSER_ACTION_OPEN,
				       GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
				       GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
				       NULL);
  gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dialog), path_selection);
  gtk_file_chooser_set_select_multiple(GTK_FILE_CHOOSER(dialog), TRUE);
  if(gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT){
    GSList*		list;
    lamipPlaylist*	playlist;
    lamipPlaylist*	playlist_tmp = 0;

    list = gtk_file_chooser_get_filenames(GTK_FILE_CHOOSER (dialog));
    /* check the playlist add the list to the list of playlist if necessary */
    if(list){
      lamip_stop();
      playlist = playlist_selected;
      if(!playlist){
	playlist = lamip_playlist_new();
	//lamip_playlist_set(playlist, 0, 0);
	playlist_add_item(playlist);
      }      
    }else{
      gtk_widget_destroy (dialog);
      return;
    }
    /* now add files to the playlist */
    playlist_tmp = playlist_selected;
    playlist_selected = playlist;
    while(list){
      char*     filename;

      filename = list->data;
      if(g_file_test(filename, G_FILE_TEST_IS_DIR)){
        _add_files_recursively(filename, playlist);
      }else{
	_add_file(filename);
      }
      //g_free (filename);
      list = g_slist_next(list);
    }
    g_slist_free(list);
    playlist_selected = playlist_tmp;
  }
  if(path_selection)
    g_free(path_selection);
  path_selection = gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER(dialog));
  gtk_widget_destroy(dialog);
}
