/*
 * the plugin managment window.
 *
 *
 *
*/

#include <gtk/gtk.h>
#include "libgtk2.h"
#include "plug_control.h"
#include "plugin_win.h"

#define PLUGIN_WIN_WIDTH	320
#define PLUGIN_WIN_HEIGHT	200

static GtkWidget*	pluginwin		= NULL;
static gboolean		config_plugin_show	= FALSE;
static GtkWidget*	plugin_treeview		= NULL;
static GtkListStore*	plugin_store		= NULL;
static gint		plugin_Type		= -1;
static GtkWidget*	chooseplugin		= NULL;
static GtkListStore*	chooseplugin_store	= NULL;
static lamipPluginHandle* handle_plugin_selected = (void*)-1;


//void		enablebutton_cb(GtkToggleButton *, gpointer);

static void cb_plugin_chooseplugin(GtkOptionMenu *, gpointer);
static void cb_plugin_value_edited(GtkCellRendererText *cell,
				   const gchar         *path_string,
				   const gchar         *new_text,
				   gpointer             data);
static void cb_plugin_plugTypeTree_selection_changed(GtkTreeSelection*,
						     gpointer);

/****************************************************************************/
static void cb_plugin_value_edited(GtkCellRendererText*	cell,
				   const gchar*		path_string,
				   const gchar*		new_text,
				   gpointer		data)
{
  GtkTreeModel*	model	= 0;
  GtkTreePath*	path	= 0;;
  GtkTreeIter	iter;
  gchar**	keys	= 0;
  char*		value	= 0;

  model = (GtkTreeModel*)data;
  path = gtk_tree_path_new_from_string(path_string);
  gtk_tree_model_get_iter(model, &iter, path);
  gtk_list_store_set(GTK_LIST_STORE(model), &iter, 1, new_text, -1);
  if(handle_plugin_selected == (void*)-1){ /*we are in general configuration*/
    keys = lamip_cfg_getCoreKeys();
    value = lamip_cfg_coreGet(keys[atoi(path_string)]);
    lamip_cfg_coreSet(keys[atoi(path_string)], new_text);
  }else{
    /* change the treeview */
    keys = lamip_cfg_getKeys(handle_plugin_selected);
    value = lamip_cfg_get(handle_plugin_selected, keys[atoi(path_string)]);
    lamip_cfg_set(handle_plugin_selected, keys[atoi(path_string)], new_text);
  }
#ifdef DEBUGING
  g_printerr("edited (%s/ (%s)->(%s))\n", path_string, value, new_text);
#endif
  g_free(value);
  g_strfreev(keys);
}

static void		cb_button_reload(GtkWidget* w, gint val)
{
  DEBUG("GTK2: reload selected plugin");

  if(handle_plugin_selected == (void*)-1)
    return;
  lamip_stop();
  lamip_plugin_deactivate(handle_plugin_selected);
  lamip_plugin_activate(handle_plugin_selected);
  lamip_play();
}


/*
 * create the window plugin managment
*/
gboolean	plugin_win_create(void)
{
  GtkWidget*		hbox;
  GtkWidget*		vbox;
  GtkWidget*		vbox2;
  GtkWidget*		button;
  GtkWidget*		frame;
  GtkCellRenderer*	cell_renderer;
  GtkTreeViewColumn*	column;
  GtkWidget*		plugTypeTree;
  GtkTreeStore*		plugTypeStore;
  GtkCellRenderer*	plugTypeRenderer;
  GtkTreeViewColumn*	plugTypeColumn;
  GtkTreeSelection*	plugTypeSelect;
  GtkTreeIter		iter1; /*root node*/
  GtkTreeIter		iter2;
  GtkCellRenderer* 	renderer;

  if(pluginwin)
    return(FALSE);

  pluginwin = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title(GTK_WINDOW(pluginwin),
		       "LAMIP gtk+2 control - plugins managment");
  gtk_window_set_default_size(GTK_WINDOW(pluginwin),
			      PLUGIN_WIN_WIDTH,
			      PLUGIN_WIN_HEIGHT);

  g_signal_connect(pluginwin, "delete-event",
		   G_CALLBACK(cb_windowplugin_show),
		   NULL);

  hbox = gtk_hbox_new(FALSE, 5);
  gtk_container_add(GTK_CONTAINER(pluginwin), hbox);

  /* creating the treeview options list */

  plugTypeStore = gtk_tree_store_new(1, G_TYPE_STRING);
  gtk_tree_store_append(plugTypeStore, &iter1, NULL);
  gtk_tree_store_set(plugTypeStore, &iter1, 0, "General", -1);
  gtk_tree_store_append(plugTypeStore, &iter1, NULL);
  gtk_tree_store_set(plugTypeStore, &iter1, 0, "Plugins", -1);

  gtk_tree_store_append(plugTypeStore, &iter2, &iter1);
  gtk_tree_store_set(plugTypeStore, &iter2, 0, "Input", -1);
  gtk_tree_store_append(plugTypeStore, &iter2, &iter1);
  gtk_tree_store_set(plugTypeStore, &iter2, 0, "Output", -1);
  gtk_tree_store_append(plugTypeStore, &iter2, &iter1);
  gtk_tree_store_set(plugTypeStore, &iter2, 0, "Control", -1);
  gtk_tree_store_append(plugTypeStore, &iter2, &iter1);
  gtk_tree_store_set(plugTypeStore, &iter2, 0, "DSP", -1);

  plugTypeTree = gtk_tree_view_new_with_model(GTK_TREE_MODEL(plugTypeStore));
  g_object_unref(G_OBJECT(plugTypeStore));
  gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(plugTypeTree), FALSE);

  plugTypeRenderer = gtk_cell_renderer_text_new();
  plugTypeColumn =
    gtk_tree_view_column_new_with_attributes("Options", plugTypeRenderer,
					     "text", 0, NULL);
  gtk_tree_view_append_column(GTK_TREE_VIEW(plugTypeTree), plugTypeColumn);

  plugTypeSelect = gtk_tree_view_get_selection(GTK_TREE_VIEW(plugTypeTree));
  gtk_tree_selection_set_mode(plugTypeSelect, GTK_SELECTION_SINGLE);
  g_signal_connect(G_OBJECT(plugTypeSelect), "changed",
		   G_CALLBACK(cb_plugin_plugTypeTree_selection_changed),
		   NULL);
  gtk_box_pack_start(GTK_BOX(hbox), plugTypeTree, FALSE, TRUE, 5);


  /* create the choose plugins options box */
  vbox = gtk_vbox_new(FALSE, 5);
  gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 5);

  chooseplugin_store = gtk_list_store_new(1, G_TYPE_STRING);
  chooseplugin =
    gtk_combo_box_new_with_model(GTK_TREE_MODEL(chooseplugin_store));
  renderer = gtk_cell_renderer_text_new();
  gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(chooseplugin), renderer, TRUE);
  gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(chooseplugin), renderer,
				 "text", 0, NULL);
  g_signal_connect(GTK_OBJECT(chooseplugin), "changed",
		   G_CALLBACK(cb_plugin_chooseplugin), NULL);
  gtk_box_pack_start(GTK_BOX(vbox), chooseplugin, FALSE, TRUE, 5);


  /* create the plugin managment frame */
  frame = gtk_frame_new("Plugin variables");
  gtk_box_pack_start(GTK_BOX(vbox), frame, TRUE, TRUE, 5);

  vbox2 = gtk_vbox_new(FALSE, 5);
  gtk_container_add(GTK_CONTAINER(frame), vbox2);


  /* create the list of variable for the plugin */
  plugin_store = gtk_list_store_new(3,
				    G_TYPE_STRING,
				    G_TYPE_STRING,
				    G_TYPE_BOOLEAN);
  plugin_treeview = gtk_tree_view_new_with_model(GTK_TREE_MODEL(plugin_store));
  gtk_container_add(GTK_CONTAINER(vbox2), plugin_treeview);

  cell_renderer=gtk_cell_renderer_text_new();
  column=gtk_tree_view_column_new_with_attributes("Plugins keys",
						  cell_renderer,
						  "text", 0,
						  NULL);
  gtk_tree_view_append_column(GTK_TREE_VIEW(plugin_treeview), column);
   
  cell_renderer=gtk_cell_renderer_text_new();
  column=gtk_tree_view_column_new_with_attributes("Plugins values",
						  cell_renderer,
						  "text", 1,
						  "editable", 2, 
						  NULL);
  g_signal_connect (cell_renderer, "edited",
		    G_CALLBACK (cb_plugin_value_edited), plugin_store);
  gtk_tree_view_append_column(GTK_TREE_VIEW(plugin_treeview), column);
    
  /* add buttons */
  hbox = gtk_hbox_new(FALSE, 0);
  gtk_box_pack_end(GTK_BOX(vbox), hbox, FALSE, FALSE, 4);
  button = gtk_button_new_with_label("Reload plugin");
  g_signal_connect(button, "clicked",
		   G_CALLBACK(cb_button_reload), NULL);
  gtk_box_pack_start(GTK_BOX (hbox), button, TRUE, TRUE, 0);
  button = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
  g_signal_connect(button, "clicked",
		   G_CALLBACK(cb_windowplugin_show), NULL);
  gtk_box_pack_start(GTK_BOX (hbox), button, TRUE, TRUE, 4);
  return(TRUE);
}

/*
 * clear the window plugin managment
 */
void	plugin_win_clean(void)
{
  if(pluginwin!=NULL){
    gtk_widget_destroy(pluginwin);
    pluginwin = NULL;
  }
}

/*
 * switch on/off the window plugin managment view
 */
int		cb_windowplugin_show(GtkWidget* w, gint val)
{
  if(config_plugin_show==FALSE){
    gtk_widget_show_all(pluginwin);
    config_plugin_show=TRUE;
  }else{
    gtk_widget_hide(pluginwin);
    config_plugin_show=FALSE;
  }
  return(TRUE);
}


/* clear key/value list
 * add values for this plugin
 */
void	cb_plugin_chooseplugin(GtkOptionMenu *optionmenu, gpointer user_data)
{
  gint			plugin_number = 0;
  lamipPluginHandle*	handle = 0;
  gint			i = 0;
  gchar**		keys = 0;

  DEBUG("GTK2: chooseplugin cb");
  plugin_number = gtk_combo_box_get_active(GTK_COMBO_BOX(chooseplugin));
  gtk_list_store_clear(plugin_store);  
  if(plugin_Type==-1)
    return;
  handle = lamip_plugin_getHandle(plugin_Type);
  i=0;
  /* set the good handle */
  while(handle && i!=plugin_number){
    handle = lamip_plugin_getNext(handle);
    i++;
  }
  handle_plugin_selected = handle;
  keys = lamip_cfg_getKeys(handle);
  if(!keys){
    DEBUG("GTK2: no configuration keys for plugin\n");
    return;
  }
  for(i=0;keys[i]!=0;i++){
    gchar*	value;
    GtkTreeIter	iter;

    value = lamip_cfg_get(handle, keys[i]);
    gtk_list_store_append(plugin_store, &iter);
    gtk_list_store_set(plugin_store, &iter,
		       0, keys[i],
		       1, value,
		       2, TRUE,
		       -1);
    g_free(value);
  }
  g_strfreev(keys);
}

static void cb_plugin_plugTypeTree_selection_changed(GtkTreeSelection* selection, gpointer data)
{
  GtkTreeIter	iter;
  GtkTreeModel*	model = 0;
  gchar*	value = 0;

  DEBUG("GTK2: plugTypeTree selection changed");
  if(gtk_tree_selection_get_selected(selection, &model, &iter)){
    lamipPluginHandle*	h_plug = 0;
    GtkTreeIter		iter2;

    gtk_tree_model_get(model, &iter, 0, &value, -1);
#ifdef DEBUGING
    g_printerr("GTK2: selected : (%s)\n", value);
#endif
    if(!g_strcasecmp("Input", value)){
      plugin_Type = PLUGIN_TYPE_INPUT;
    }else if(!g_strcasecmp("Output", value)){
      plugin_Type = PLUGIN_TYPE_OUTPUT;
    }else if(!g_strcasecmp("Control", value)){
      plugin_Type = PLUGIN_TYPE_CONTROL;
    }else if(!g_strcasecmp("DSP", value)){
      plugin_Type = PLUGIN_TYPE_DSP;
    }else if(!g_strcasecmp("General", value)){
      gchar**	keys	= 0;
      int	i	= 0;

      gtk_list_store_clear(chooseplugin_store);
      gtk_list_store_clear(plugin_store);
      handle_plugin_selected = (void*)-1;
      keys = lamip_cfg_getCoreKeys();
      for(i=0;keys[i]!=0;i++){
	gchar*		value2 = 0;
	GtkTreeIter	iter;

	value2 = lamip_cfg_coreGet(keys[i]);
	gtk_list_store_append(plugin_store, &iter);
	gtk_list_store_set(plugin_store, &iter,
			   0, keys[i],
			   1, value2,
			   2, TRUE,
			   -1);
	g_free(value2);
      }
      g_free(value);
      g_strfreev(keys);
      return;
    }else{
      handle_plugin_selected = (void*)-1;
      gtk_list_store_clear(chooseplugin_store);
      gtk_list_store_clear(plugin_store);
      g_free(value);
      plugin_Type = -1;
      return;
    }
    /* we are here in a plugin type */
    g_free(value);
    h_plug = lamip_plugin_getHandle(plugin_Type);
    handle_plugin_selected = (void*)-1;
    gtk_list_store_clear(chooseplugin_store);
    gtk_list_store_clear(plugin_store);
    while(h_plug!=NULL){
      gchar*		buf = lamip_plugin_getName(h_plug);

      gtk_list_store_append(chooseplugin_store, &iter2);
      gtk_list_store_set(chooseplugin_store, &iter2, 0, buf, -1);
      g_free(buf);
      h_plug = lamip_plugin_getNext(h_plug);
    }
  }
}
