/*
 * This window show you a list of the shoutcast live stream radio
 * it's the same window as winamp media library
 *
*/

#ifdef WITH_SHOUTCAST

#include <gtk/gtk.h>
#include <curl/curl.h>
#include <stdlib.h>
#include <string.h>

#include "lamip.h"
#include "plug_control.h"
#include "playlist_win.h"
#include "stream_win.h"

// shoutcast enums
enum {
  STREAM_ICY_NAME,
  STREAM_TITLE,
  STREAM_GENRE,
  STREAM_LISTENERS,
  STREAM_BITRATE,
  N_STREAM_COLUMNS
};

typedef enum {
  XML_PLAYLIST,
   XML_ENTRY,
    XML_NAME,
    XML_GENRE,
    XML_NOWPLAYING,
    XML_LISTERNERS,
    XML_BITRATE,
  XML_UNKNOW
} xml_parserState;

// icecast enums
enum {
  ICE_LISTENURL,
  ICE_SERVERNAME,
  ICE_BITRATE,
  ICE_GENRE,
  ICE_CHANNELS,
  ICE_STREAMTYPE,
  N_ICE_COLUMNS
};

typedef enum {
  XML_ICE_DIRECTORY,
   XML_ICE_ENTRY,
    XML_ICE_SERVERNAME,
    XML_ICE_LISTENURL,
    XML_ICE_SERVERTYPE,
    XML_ICE_BITRATE,
    XML_ICE_CHANNELS,
    XML_ICE_SAMPLERATE,
    XML_ICE_GENRE,
    XML_ICE_CURRENTSONG,
  XML_ICE_UNKNOW
} xml_icecastState;

// peercast enums
enum {
  PEER_ID,
  PEER_CHANNEL_NAME,
  PEER_GENRE,
  PEER_TYPE,
  PEER_BITRATE,
  PEER_AGE,
  PEER_COMMENT,
  N_PEER_COLUMNS
};

typedef enum {
  XML_PEER_PEERCAST,
   XML_PEER_SERVENT,
   XML_PEER_BANDWIDTH,
   XML_PEER_CONNECTIONS,
   XML_PEER_CHANNELS_RELAYED,
   XML_PEER_CHANNELS_FOUND,
    XML_PEER_CHANNEL,
     XML_PEER_HITS,
      XML_PEER_HOST,
     XML_PEER_TRACK,
  XML_PEER_UNKNOW
} xml_peercastState;

#define STREAM_DEFAULT_WIDTH	450
#define STREAM_DEFAULT_HEIGHT	250

/* !!! hack hack hack -> shoutcast people don't want other software than
 * winamp do this, it's just come from sniffing winamp :-p
 * but well it's working pretty well
*/
#define SHOUTCAST_LIST	"http://www.shoutcast.com/sbin/xmllister.phtml?service=winamp2&limit=500&no_compress=1"
#define ICECAST_LIST	"http://dir.xiph.org/yp.xml"
#define PEERCAST_LIST	"http://yp.peercast.org/?xml"

static GtkWidget*	stream_win		= NULL;
static gboolean		config_stream_show	= FALSE;

static GtkTreeModel*	stream_model		= NULL;
static GtkWidget*	stream_treeview		= NULL;

static GtkTreeModel*	icecast_model		= NULL;
static GtkWidget*	icecast_treeview	= NULL;

static GtkTreeModel*	peercast_model		= NULL;
static GtkWidget*	peercast_treeview	= NULL;
static gchar*		peercast_hostname	= 0;

static GtkTreeModel*	stream_create_model(void);
static GtkTreeModel*	icecast_create_model(void);
static GtkTreeModel*	peercast_create_model(void);

static gboolean		creating_shoutcastlist(void);
static gboolean		creating_icecastlist(void);
static gboolean		creating_peercastlist(void);

static void		cleaning_shoutcastlist(void);
static void		cleaning_icecastlist(void);
static void		cleaning_peercastlist(void);

static size_t		_curl_write2buffer(void*, size_t, size_t, void*);


static void parser_start_element(GMarkupParseContext*,
				 const gchar*, const gchar**, const gchar**,
				 gpointer, GError**);
static void parser_end_element(GMarkupParseContext*, const gchar*,
			       gpointer, GError**);
static void parser_text_element(GMarkupParseContext*,
				const gchar*, gsize, gpointer, GError**);

static void ice_parser_start_element(GMarkupParseContext*, const gchar*,
				     const gchar**, const gchar**,
				     gpointer, GError**);
static void ice_parser_end_element(GMarkupParseContext*, const gchar*,
				   gpointer, GError**);
static void ice_parser_text_element(GMarkupParseContext*,
				    const gchar*, gsize, gpointer, GError**);

static void peer_parser_start_element(GMarkupParseContext*, const gchar*,
				      const gchar**, const gchar**,
				      gpointer, GError**);
static void peer_parser_end_element(GMarkupParseContext*, const gchar*,
				    gpointer, GError**);
static void peer_parser_text_element(GMarkupParseContext*,
				     const gchar*, gsize, gpointer, GError**);



static gboolean	cb_stream_treeview_selected(GtkWidget*, GdkEventButton*,
					    gpointer);
static gboolean	cb_icecast_treeview_selected(GtkWidget*, GdkEventButton*,
					     gpointer);
static gboolean	cb_peercast_treeview_selected(GtkWidget*, GdkEventButton*,
					      gpointer);

static void	cb_shoutcastwin_refresh(GtkWidget*, gint);
static void	cb_shoutcastwin_refresh_pressed(GtkWidget*, gint);
static void	cb_icecastwin_refresh(GtkWidget*, gint);
static void	cb_icecastwin_refresh_pressed(GtkWidget*, gint);
static void	cb_peercastwin_refresh(GtkWidget*, gint);
static void	cb_peercastwin_refresh_pressed(GtkWidget*, gint);
static void	cb_peercast_hostname_entry(GtkWidget*, gint);

/****************************************************************************/
gboolean	windowstream_create(void)
{
  GtkWidget*		vbox;
  GtkWidget*		hbox;
  GtkWidget*		vvbox;
  GtkWidget*		sw;
  GtkCellRenderer*	cell_renderer;
  GtkTreeViewColumn*	column;
  GtkTreeSelection*	stream_select;
  GtkTreeSelection*	icecast_select;
  GtkTreeSelection*	peercast_select;
  GtkWidget*		button;
  GtkWidget*		notebook;
  GtkWidget*		label1;
  GtkWidget*		peercast_hostname_entry;

  if(stream_win)
    return(TRUE);
  stream_win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title(GTK_WINDOW(stream_win),
		       "LAMIP gtk+2 control gui - streams list");  
  /*
  g_signal_connect(G_OBJECT(stream_win),
		   "destroy",
		   G_CALLBACK(stream_clean),
		   &stream_win);
  */
  g_signal_connect(G_OBJECT(stream_win),
		   "delete-event",
		   G_CALLBACK(cb_windowstream_show),
		   NULL);
  vvbox = gtk_vbox_new(FALSE, 0);
  gtk_container_add(GTK_CONTAINER(stream_win), vvbox);

  notebook = gtk_notebook_new();
  gtk_notebook_set_tab_pos(GTK_NOTEBOOK(notebook), GTK_POS_LEFT);
  gtk_box_pack_start(GTK_BOX(vvbox), notebook, TRUE, TRUE, 0);

  /*
   * Page 1 : shoutcast list
  */

  vbox = gtk_vbox_new(FALSE, 0);
  label1 = gtk_label_new("Shoutcast");
  
  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox, label1);
// add 'refresh list' button
  button = gtk_button_new_from_stock(GTK_STOCK_REFRESH);
  g_signal_connect (button, "clicked",
		    G_CALLBACK(cb_shoutcastwin_refresh), NULL);
  g_signal_connect (button, "pressed",
		    G_CALLBACK(cb_shoutcastwin_refresh_pressed), NULL);
  gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 0);

// the list of shoutcast streams
  sw = gtk_scrolled_window_new(NULL, NULL);
  gtk_box_pack_start(GTK_BOX(vbox), sw, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
				 GTK_POLICY_AUTOMATIC,
				 GTK_POLICY_AUTOMATIC);
  stream_model = stream_create_model();
  stream_treeview = gtk_tree_view_new_with_model(stream_model);

  stream_select = gtk_tree_view_get_selection(GTK_TREE_VIEW(stream_treeview));
  gtk_tree_selection_set_mode(stream_select, GTK_SELECTION_SINGLE);
  g_signal_connect(G_OBJECT(stream_treeview), "button_press_event",
		   G_CALLBACK(cb_stream_treeview_selected),
		   GTK_TREE_MODEL(stream_model));


  gtk_container_add(GTK_CONTAINER(sw), stream_treeview);
  /*
  cell_renderer = gtk_cell_renderer_text_new();
  column = gtk_tree_view_column_new_with_attributes("Shoutcast Live !",
						    cell_renderer,
						    "text",
						    STREAM_ICY_NAME,
						    NULL);
  gtk_tree_view_append_column(GTK_TREE_VIEW(stream_treeview), column);
  */
  cell_renderer = gtk_cell_renderer_text_new();
  column = gtk_tree_view_column_new_with_attributes("Title",
						    cell_renderer,
						    "text",
						    STREAM_TITLE,
						    NULL);
  gtk_tree_view_column_set_sort_column_id(column, STREAM_TITLE);
  gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_FIXED);
  gtk_tree_view_column_set_fixed_width(column, 120);
  gtk_tree_view_column_set_resizable(column, TRUE);
  gtk_tree_view_append_column(GTK_TREE_VIEW(stream_treeview), column);
  cell_renderer = gtk_cell_renderer_text_new();
  column = gtk_tree_view_column_new_with_attributes("Genre",
						    cell_renderer,
						    "text",
						    STREAM_GENRE,
						    NULL);
  gtk_tree_view_column_set_sort_column_id(column, STREAM_GENRE);
  gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_FIXED);
  gtk_tree_view_column_set_fixed_width(column, 100);
  gtk_tree_view_column_set_resizable(column, TRUE);
  gtk_tree_view_append_column(GTK_TREE_VIEW(stream_treeview), column);

  cell_renderer = gtk_cell_renderer_text_new();
  column = gtk_tree_view_column_new_with_attributes("Listeners",
						    cell_renderer,
						    "text",
						    STREAM_LISTENERS,
						    NULL);
  gtk_tree_view_column_set_sort_column_id(column, STREAM_LISTENERS);
  //gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_FIXED);
  //gtk_tree_view_column_set_fixed_width(column, 50);
  gtk_tree_view_column_set_resizable(column, TRUE);
  gtk_tree_view_append_column(GTK_TREE_VIEW(stream_treeview), column);

  cell_renderer = gtk_cell_renderer_text_new();
  column = gtk_tree_view_column_new_with_attributes("Bitrate",
						    cell_renderer,
						    "text",
						    STREAM_BITRATE,
						    NULL);
  gtk_tree_view_column_set_sort_column_id(column, STREAM_BITRATE);
  //gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_FIXED);
  //gtk_tree_view_column_set_fixed_width(column, 50);
  gtk_tree_view_column_set_resizable(column, TRUE);
  gtk_tree_view_append_column(GTK_TREE_VIEW(stream_treeview), column);


  /*
   * page 2 : icecast streams
  */

  vbox = gtk_vbox_new(FALSE, 0);
  label1 = gtk_label_new("Icecast (xiph)");  
  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox, label1);
// add 'refresh list' button
  button = gtk_button_new_from_stock(GTK_STOCK_REFRESH);
  
  g_signal_connect (button, "clicked",
		    G_CALLBACK(cb_icecastwin_refresh), NULL);
  g_signal_connect (button, "pressed",
		    G_CALLBACK(cb_icecastwin_refresh_pressed), NULL);
  
  gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 0);

// the list of icecast streams
  sw = gtk_scrolled_window_new(NULL, NULL);
  gtk_box_pack_start(GTK_BOX(vbox), sw, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
				 GTK_POLICY_AUTOMATIC,
				 GTK_POLICY_AUTOMATIC);
  icecast_model = icecast_create_model();
  icecast_treeview = gtk_tree_view_new_with_model(icecast_model);

  icecast_select = gtk_tree_view_get_selection(GTK_TREE_VIEW(icecast_treeview));
  gtk_tree_selection_set_mode(icecast_select, GTK_SELECTION_SINGLE);
  g_signal_connect(G_OBJECT(icecast_treeview), "button_press_event",
		   G_CALLBACK(cb_icecast_treeview_selected),
		   GTK_TREE_MODEL(icecast_model));

  gtk_container_add(GTK_CONTAINER(sw), icecast_treeview);
  
  cell_renderer = gtk_cell_renderer_text_new();
  column = gtk_tree_view_column_new_with_attributes("Title",
						    cell_renderer, "text",
						    ICE_SERVERNAME, NULL);
  gtk_tree_view_column_set_sort_column_id(column, ICE_SERVERNAME);
  gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_FIXED);
  gtk_tree_view_column_set_fixed_width(column, 120);
  gtk_tree_view_column_set_resizable(column, TRUE);
  gtk_tree_view_append_column(GTK_TREE_VIEW(icecast_treeview), column);

  cell_renderer = gtk_cell_renderer_text_new();
  column = gtk_tree_view_column_new_with_attributes("Genre",
						    cell_renderer, "text",
						    ICE_GENRE, NULL);
  gtk_tree_view_column_set_sort_column_id(column, ICE_GENRE);
  gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_FIXED);
  gtk_tree_view_column_set_fixed_width(column, 100);
  gtk_tree_view_column_set_resizable(column, TRUE);
  gtk_tree_view_append_column(GTK_TREE_VIEW(icecast_treeview), column);

  cell_renderer = gtk_cell_renderer_text_new();
  column = gtk_tree_view_column_new_with_attributes("Type",
						    cell_renderer, "text",
						    ICE_STREAMTYPE, NULL);
  gtk_tree_view_column_set_sort_column_id(column, ICE_STREAMTYPE);
  //gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_FIXED);
  //gtk_tree_view_column_set_fixed_width(column, 50);
  gtk_tree_view_column_set_resizable(column, TRUE);
  gtk_tree_view_append_column(GTK_TREE_VIEW(icecast_treeview), column);

  cell_renderer = gtk_cell_renderer_text_new();
  column = gtk_tree_view_column_new_with_attributes("Bitrate",
						    cell_renderer, "text",
						    ICE_BITRATE, NULL);
  gtk_tree_view_column_set_sort_column_id(column, ICE_BITRATE);
  //gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_FIXED);
  //gtk_tree_view_column_set_fixed_width(column, 50);
  gtk_tree_view_column_set_resizable(column, TRUE);
  gtk_tree_view_append_column(GTK_TREE_VIEW(icecast_treeview), column);

  /*
   *
   * page 3 : peercast streams
  */

  vbox = gtk_vbox_new(FALSE, 0);
  label1 = gtk_label_new("Peercast");  
  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox, label1);
// add 'refresh list' button and the hostname entry
  hbox = gtk_hbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
  label1 = gtk_label_new("Hostname:");
  gtk_box_pack_start(GTK_BOX(hbox), label1, FALSE, FALSE, 0);
  peercast_hostname_entry = gtk_entry_new();

  g_signal_connect(peercast_hostname_entry, "activate",
		   G_CALLBACK(cb_peercast_hostname_entry), NULL);

  gtk_box_pack_start(GTK_BOX(hbox), peercast_hostname_entry, FALSE, FALSE, 0);
  button = gtk_button_new_from_stock(GTK_STOCK_REFRESH);
  
  g_signal_connect (button, "clicked",
		    G_CALLBACK(cb_peercastwin_refresh), NULL);
  g_signal_connect (button, "pressed",
		    G_CALLBACK(cb_peercastwin_refresh_pressed), NULL);
  
  gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);

// the list of peercast streams
  sw = gtk_scrolled_window_new(NULL, NULL);
  gtk_box_pack_start(GTK_BOX(vbox), sw, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
				 GTK_POLICY_AUTOMATIC,
				 GTK_POLICY_AUTOMATIC);
  peercast_model = peercast_create_model();
  peercast_treeview = gtk_tree_view_new_with_model(peercast_model);

  peercast_select = gtk_tree_view_get_selection(GTK_TREE_VIEW(peercast_treeview));
  gtk_tree_selection_set_mode(peercast_select, GTK_SELECTION_SINGLE);
  g_signal_connect(G_OBJECT(peercast_treeview), "button_press_event",
		   G_CALLBACK(cb_peercast_treeview_selected),
		   GTK_TREE_MODEL(peercast_model));

  gtk_container_add(GTK_CONTAINER(sw), peercast_treeview);
  
  cell_renderer = gtk_cell_renderer_text_new();
  column = gtk_tree_view_column_new_with_attributes("Title",
						    cell_renderer, "text",
						    PEER_CHANNEL_NAME, NULL);
  gtk_tree_view_column_set_sort_column_id(column, PEER_CHANNEL_NAME);
  gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_FIXED);
  gtk_tree_view_column_set_fixed_width(column, 120);
  gtk_tree_view_column_set_resizable(column, TRUE);
  gtk_tree_view_append_column(GTK_TREE_VIEW(peercast_treeview), column);

  cell_renderer = gtk_cell_renderer_text_new();
  column = gtk_tree_view_column_new_with_attributes("Genre",
						    cell_renderer, "text",
						    PEER_GENRE, NULL);
  gtk_tree_view_column_set_sort_column_id(column, PEER_GENRE);
  gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_FIXED);
  gtk_tree_view_column_set_fixed_width(column, 100);
  gtk_tree_view_column_set_resizable(column, TRUE);
  gtk_tree_view_append_column(GTK_TREE_VIEW(peercast_treeview), column);

  cell_renderer = gtk_cell_renderer_text_new();
  column = gtk_tree_view_column_new_with_attributes("Type",
						    cell_renderer, "text",
						    PEER_TYPE, NULL);
  gtk_tree_view_column_set_sort_column_id(column, PEER_TYPE);
  //gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_FIXED);
  //gtk_tree_view_column_set_fixed_width(column, 50);
  gtk_tree_view_column_set_resizable(column, TRUE);
  gtk_tree_view_append_column(GTK_TREE_VIEW(peercast_treeview), column);

  cell_renderer = gtk_cell_renderer_text_new();
  column = gtk_tree_view_column_new_with_attributes("Bitrate",
						    cell_renderer, "text",
						    PEER_BITRATE, NULL);
  gtk_tree_view_column_set_sort_column_id(column, PEER_BITRATE);
  //gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_FIXED);
  //gtk_tree_view_column_set_fixed_width(column, 50);
  gtk_tree_view_column_set_resizable(column, TRUE);
  gtk_tree_view_append_column(GTK_TREE_VIEW(peercast_treeview), column);

  cell_renderer = gtk_cell_renderer_text_new();
  column = gtk_tree_view_column_new_with_attributes("Age",
						    cell_renderer, "text",
						    PEER_AGE, NULL);
  gtk_tree_view_column_set_sort_column_id(column, PEER_AGE);
  //gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_FIXED);
  //gtk_tree_view_column_set_fixed_width(column, 50);
  gtk_tree_view_column_set_resizable(column, TRUE);
  gtk_tree_view_append_column(GTK_TREE_VIEW(peercast_treeview), column);

  cell_renderer = gtk_cell_renderer_text_new();
  column = gtk_tree_view_column_new_with_attributes("Comment",
						    cell_renderer, "text",
						    PEER_COMMENT, NULL);
  gtk_tree_view_column_set_sort_column_id(column, PEER_COMMENT);
  //gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_FIXED);
  //gtk_tree_view_column_set_fixed_width(column, 50);
  gtk_tree_view_column_set_resizable(column, TRUE);
  gtk_tree_view_append_column(GTK_TREE_VIEW(peercast_treeview), column);


  gtk_widget_set_size_request(stream_win,
			      STREAM_DEFAULT_WIDTH, STREAM_DEFAULT_HEIGHT);
  return(TRUE);
}

void		windowstream_clean(void)
{
  gtk_list_store_clear(GTK_LIST_STORE(stream_model));
  gtk_list_store_clear(GTK_LIST_STORE(icecast_model));
  gtk_list_store_clear(GTK_LIST_STORE(peercast_model));
  g_object_unref(G_OBJECT(stream_model));
  g_object_unref(G_OBJECT(icecast_model));
  g_object_unref(G_OBJECT(peercast_model));
  if(stream_win) gtk_widget_destroy(stream_win);
  stream_win = NULL;
}

int		cb_windowstream_show(GtkWidget* w, gint val)
{
  if(config_stream_show==FALSE){
    gtk_widget_show_all(stream_win);
    config_stream_show = TRUE;
  }else{
    gtk_widget_hide(stream_win);
    config_stream_show = FALSE;
    cleaning_shoutcastlist();
    cleaning_icecastlist();
    cleaning_peercastlist();
  }
  return(TRUE);
}

static void	cb_shoutcastwin_refresh_pressed(GtkWidget* w, gint val)
{
  gtk_button_set_label(GTK_BUTTON(w), "Creating shoutcast list...");
  gdk_flush();
}
static void	cb_shoutcastwin_refresh(GtkWidget* w, gint val)
{
  GtkWidget*            dialog;

  gtk_button_set_label(GTK_BUTTON(w), GTK_STOCK_REFRESH);
  if(creating_shoutcastlist()==FALSE)
  {
    dialog = gtk_message_dialog_new (GTK_WINDOW(stream_win),
				     GTK_DIALOG_DESTROY_WITH_PARENT,
				     GTK_MESSAGE_ERROR,
				     GTK_BUTTONS_CLOSE,
				     "Can't retrieve list!");
    gtk_dialog_run (GTK_DIALOG (dialog));
    gtk_widget_destroy (dialog);
  }
}

static void	cb_icecastwin_refresh_pressed(GtkWidget* w, gint val)
{
  gtk_button_set_label(GTK_BUTTON(w), "Creating icecast list...");
  gdk_flush();
}
static void	cb_icecastwin_refresh(GtkWidget* w, gint val)
{
  GtkWidget*            dialog;

  gtk_button_set_label(GTK_BUTTON(w), GTK_STOCK_REFRESH);
  if(creating_icecastlist()==FALSE)
  {
    dialog = gtk_message_dialog_new (GTK_WINDOW(stream_win),
				     GTK_DIALOG_DESTROY_WITH_PARENT,
				     GTK_MESSAGE_ERROR,
				     GTK_BUTTONS_CLOSE,
				     "Can't retrieve list!");
    gtk_dialog_run (GTK_DIALOG (dialog));
    gtk_widget_destroy (dialog);
  }
}

static void	cb_peercastwin_refresh_pressed(GtkWidget* w, gint val)
{
  gtk_button_set_label(GTK_BUTTON(w), "Creating peercast list...");
  gdk_flush();
}
static void	cb_peercastwin_refresh(GtkWidget* w, gint val)
{
  GtkWidget*            dialog;

  gtk_button_set_label(GTK_BUTTON(w), GTK_STOCK_REFRESH);
  if(creating_peercastlist()==FALSE)
  {
    dialog = gtk_message_dialog_new (GTK_WINDOW(stream_win),
				     GTK_DIALOG_DESTROY_WITH_PARENT,
				     GTK_MESSAGE_ERROR,
				     GTK_BUTTONS_CLOSE,
				     "Can't retrieve list!");
    gtk_dialog_run (GTK_DIALOG (dialog));
    gtk_widget_destroy (dialog);
  }
}

static void	cb_peercast_hostname_entry(GtkWidget* w, gint val)
{
  if(peercast_hostname) g_free(peercast_hostname);
  peercast_hostname = (gchar*)gtk_entry_get_text(GTK_ENTRY(w));
#ifdef DEBUG
  g_printerr("GTK2: hostname set to : %s\n", peercast_hostname);
#endif
  
}

/***************************************************************************/
/*
 * SHOUTCAST PART
*/

static GtkTreeModel*	stream_create_model(void)
{
  GtkListStore*	store;

  store = gtk_list_store_new(N_STREAM_COLUMNS,
			     G_TYPE_STRING, // icy-name
			     G_TYPE_STRING, // title
			     G_TYPE_STRING, // genre
			     G_TYPE_STRING, // listerners
			     G_TYPE_STRING); //bitrate
  return(GTK_TREE_MODEL(store));
}


static char*	shoutcastbuffer = 0;
static CURL*	curl = 0;

static size_t _curl_write2buffer(void* data, size_t size, size_t nmemb, void* stream)
{
  char* tmp = malloc(size*nmemb+1);
  char* tmp2 = shoutcastbuffer;
  
  memset(tmp, 0, size*nmemb+1);
  memcpy(tmp, data, size*nmemb);
  shoutcastbuffer = g_strconcat(tmp2, tmp, NULL);
  g_print(".");
  free(tmp);
  free(tmp2);
  return(size*nmemb);
}

static GMarkupParser parserFuncs =
  {
    parser_start_element,
    parser_end_element,
    parser_text_element,
    NULL,
    NULL
  };


static gboolean		creating_shoutcastlist(void)
{
  gint ret = 0;
  GMarkupParseContext*  context;

#ifdef DEBUG
  g_printerr("GTK2: creating shoutcast list\n");
#endif
  gtk_list_store_clear(GTK_LIST_STORE(stream_model));  
  shoutcastbuffer = g_strdup("");
  curl = curl_easy_init();
  if(!curl) return(FALSE);

  curl_easy_setopt(curl, CURLOPT_URL, SHOUTCAST_LIST);
  curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, _curl_write2buffer);
  curl_easy_setopt(curl, CURLOPT_VERBOSE, 0);
  curl_easy_setopt(curl, CURLOPT_NOPROGRESS, 1);
  curl_easy_setopt(curl, CURLOPT_CONNECTTIMEOUT, 15);
  curl_easy_setopt(curl, CURLOPT_LOW_SPEED_LIMIT, 0);
  curl_easy_setopt(curl, CURLOPT_LOW_SPEED_TIME, 15);
  curl_easy_setopt(curl, CURLOPT_USERAGENT, "lamip/" VERSION);
  curl_easy_setopt(curl, CURLOPT_HTTPGET, 1);
  ret = curl_easy_perform(curl);
  if(ret != CURLE_OK){
    g_print("GTK2: Can't download the shoutcast stream list");
    return(FALSE);
  }
  /* the shoutcastlist string iss full now, let's add content
   * to the treeview
   */
#ifdef DEBUG
  g_printerr("%s", shoutcastbuffer);
#endif
  context = g_markup_parse_context_new(&parserFuncs, 0, NULL, NULL);
  g_markup_parse_context_parse(context, shoutcastbuffer, strlen(shoutcastbuffer), NULL);
  if(shoutcastbuffer) g_free(shoutcastbuffer);
  shoutcastbuffer = 0;
  return(TRUE);
}

static void		cleaning_shoutcastlist(void)
{
}


static GtkTreeIter	iter;
static xml_parserState	parser_state = XML_UNKNOW;

static void parser_start_element(GMarkupParseContext *context,
				 const gchar         *element_name,
				 const gchar        **attribute_names,
				 const gchar        **attribute_values,
				 gpointer             user_data,
				 GError             **error)
{
  //g_print("start: %s\n", element_name);

  switch(parser_state){
  case XML_UNKNOW:
    {
      if((!g_ascii_strcasecmp(element_name, "playlist"))){
	gint i;

	parser_state = XML_PLAYLIST;
	for(i=0;attribute_names && attribute_names[i]!=NULL;i+=1){
	  if(!g_ascii_strcasecmp(attribute_names[i], "label")){
	    //g_print("label : %s (%s/%d)\n", attribute_values[i],__FILE__, __LINE__);
	  }else if(!g_ascii_strcasecmp(attribute_names[i], "num_entries")){
	    //g_print("entries: %s\n", attribute_values[i]);
	  }
	}
      }
      break;
    }

  case XML_PLAYLIST:
    {
      if((!g_ascii_strcasecmp(element_name, "entry"))){
	gint i;

	parser_state = XML_ENTRY;
	for(i=0;attribute_names && attribute_names[i]!=NULL;i+=1){
	  if(!g_ascii_strcasecmp(attribute_names[i], "playstring")){
	    
	    //g_print("URL: %s\n", attribute_values[i]);
	    gtk_list_store_append(GTK_LIST_STORE(stream_model), &iter);
	    gtk_list_store_set(GTK_LIST_STORE(stream_model), &iter,
			       0, attribute_values[i],
			       -1);
	  }
	}
      }
      break;
    }
  case XML_ENTRY:
    {
      if((!g_ascii_strcasecmp(element_name, "name"))){
	parser_state = XML_NAME;
      }else if((!g_ascii_strcasecmp(element_name, "genre"))){
	parser_state = XML_GENRE;
      }else if((!g_ascii_strcasecmp(element_name, "nowplaying"))){
	parser_state = XML_NOWPLAYING;
      }else if((!g_ascii_strcasecmp(element_name, "listeners"))){
	parser_state = XML_LISTERNERS;
      }else if((!g_ascii_strcasecmp(element_name, "bitrate"))){
	parser_state = XML_BITRATE;
      }
      break;
    }
  case XML_NAME:
  case XML_GENRE:
  case XML_NOWPLAYING:
  case XML_LISTERNERS:
  case XML_BITRATE:
    {
      break;
    }
  default:
    {
      g_printerr("GTK2: You should not be here, XML is bad\n");
      parser_state = XML_UNKNOW;
      break;
    }
  }
}

static void parser_end_element(GMarkupParseContext *context,
			       const gchar         *element_name,
			       gpointer             user_data,
			       GError             **error)
{
  //g_print("end: %s\n", element_name);
  switch(parser_state){
  case XML_UNKNOW:
    {
      break;
    }
  case XML_PLAYLIST:
    {
      if((!g_ascii_strcasecmp(element_name, "playlist"))){
	parser_state = XML_UNKNOW;
      }
      break;
    }
  case XML_ENTRY:
    {
      if((!g_ascii_strcasecmp(element_name, "entry"))){
	parser_state = XML_PLAYLIST;
      }
      break;
    }
  case XML_NAME:
    {
      if((!g_ascii_strcasecmp(element_name, "name"))){
	parser_state = XML_ENTRY;
      }
      break;
    }
  case XML_GENRE:
    {
      if((!g_ascii_strcasecmp(element_name, "genre"))){
	parser_state = XML_ENTRY;
      }
      break;
    }
  case XML_NOWPLAYING:
    {
      if((!g_ascii_strcasecmp(element_name, "nowplaying"))){
	parser_state = XML_ENTRY;
      }
      break;
    }
  case XML_LISTERNERS:
    {
      if((!g_ascii_strcasecmp(element_name, "listeners"))){
	parser_state = XML_ENTRY;
      }
      break;
    }
  case XML_BITRATE:
    {
      if((!g_ascii_strcasecmp(element_name, "bitrate"))){
	parser_state = XML_ENTRY;
      }
      break;
    }
  }

}

static void parser_text_element(GMarkupParseContext *context,
                                    const gchar         *text,
                                    gsize                text_len,  
                                    gpointer             user_data,
                                    GError             **error)
{
  switch(parser_state){
  case XML_UNKNOW:
    {
      break;
    }
  case XML_PLAYLIST:
    {
      break;
    }
  case XML_ENTRY:
    {
      break;
    }
  case XML_NAME:
    {
      //g_print("stream_name: %s\n", text);
      gtk_list_store_set(GTK_LIST_STORE(stream_model), &iter,
			 STREAM_TITLE, text,
			 -1);
      break;
    }
  case XML_GENRE:
    {
      //g_print("stream_genre: %s\n", text);
      gtk_list_store_set(GTK_LIST_STORE(stream_model), &iter,
			 STREAM_GENRE, text,
			 -1);
      break;
    }
  case XML_NOWPLAYING:
    {
      //g_print("stream_playing: %s\n", text);
      break;
    }
  case XML_LISTERNERS:
    {
      //g_print("stream_listeners: %s\n", text);
      gtk_list_store_set(GTK_LIST_STORE(stream_model), &iter,
			 STREAM_LISTENERS, text,
			 -1);
      break;
    }
  case XML_BITRATE:
    {
      //g_print("stream_bitrate: %s\n", text);
      gtk_list_store_set(GTK_LIST_STORE(stream_model), &iter,
			 STREAM_BITRATE, text,
			 -1);
      break;
    }
  }
}

static gboolean	cb_stream_treeview_selected(GtkWidget* widget,
					    GdkEventButton* event,
					    gpointer data)
{
  if(event->button == 1 && event->type == GDK_2BUTTON_PRESS){
    GtkTreeSelection*	selection;
    GtkTreeIter		iter;
    GtkTreeModel*	model;

      //g_print("double clicked !!!\n");
      selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(stream_treeview));
      if(gtk_tree_selection_get_selected(selection, &model, &iter)){
	gchar*		url;
	lamipPlaylist*	playlist;
	lamipSonginfo*	info;
	GtkTreeSelection* selection2;

	selection2 =
	  gtk_tree_view_get_selection(GTK_TREE_VIEW(playlist_treeview));

	gtk_tree_model_get(model, &iter, STREAM_ICY_NAME, &url, -1);
#ifdef DEBUG
	g_printerr("You selected %s\n", url);
#endif
	lamip_stop();
	playlist_clean();
	playlist = lamip_playlist_new();
	playlist_add_item(playlist); // add to list of playlist
	info = lamip_songinfo_new();
	lamip_songinfo_read(url, info);
	lamip_playlist_append(playlist, info);
	lamip_playlist_set(playlist, 0, 0);
	g_free(url);
	// select the correct playlist
	playlist_selected = playlist;
	gtk_tree_selection_select_path(selection2,
				       gtk_tree_path_new_from_string("0"));
	lamip_play();
      }
    return(FALSE);
  }else if(event->button == 1 && event->type == GDK_BUTTON_PRESS){
    //g_print("clicked !!!\n");
    return(FALSE);
  }
  return(TRUE);
}


/***************************************************************************/
/*
 * ICECAST PART
*/

static GtkTreeModel*	icecast_create_model(void)
{
  GtkListStore*	store;

  store = gtk_list_store_new(N_ICE_COLUMNS,
			     G_TYPE_STRING, // listen url
			     G_TYPE_STRING, // servername / title
			     G_TYPE_STRING, // bitrate
			     G_TYPE_STRING, // genre
			     G_TYPE_STRING, // channels
			     G_TYPE_STRING); // streamtype (ogg or mp3)
  return(GTK_TREE_MODEL(store));
}

// we use the same buffer text of shoutcast

static GMarkupParser ice_parserFuncs =
  {
    ice_parser_start_element,
    ice_parser_end_element,
    ice_parser_text_element,
    NULL,
    NULL
  };

static gboolean		creating_icecastlist(void)
{
  gint			ret = 0;
  GMarkupParseContext*	context;

#ifdef DEBUG
  g_printerr("GTK2: creating Icecast list\n");
#endif
  gtk_list_store_clear(GTK_LIST_STORE(icecast_model));
  shoutcastbuffer = g_strdup("");
  curl = curl_easy_init();
  if(!curl) return(FALSE);

  curl_easy_setopt(curl, CURLOPT_URL, ICECAST_LIST);
  curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, _curl_write2buffer);
  curl_easy_setopt(curl, CURLOPT_VERBOSE, 0);
  curl_easy_setopt(curl, CURLOPT_NOPROGRESS, 1);
  curl_easy_setopt(curl, CURLOPT_CONNECTTIMEOUT, 15);
  curl_easy_setopt(curl, CURLOPT_LOW_SPEED_LIMIT, 0);
  curl_easy_setopt(curl, CURLOPT_LOW_SPEED_TIME, 15);
  curl_easy_setopt(curl, CURLOPT_USERAGENT, "lamip/" VERSION);
  curl_easy_setopt(curl, CURLOPT_HTTPGET, 1);
  ret = curl_easy_perform(curl);
  if(ret != CURLE_OK){
    g_printerr("GTK2: Can't download the Icecast streams list");
    return(FALSE);
  }
  /* the icecastlist string is full now, let's add content
   * to the treeview
   */
#ifdef DEBUG
  g_printerr("%s", shoutcastbuffer);
#endif
  context = g_markup_parse_context_new(&ice_parserFuncs, 0, NULL, NULL);
  if(g_markup_parse_context_parse(context, shoutcastbuffer, strlen(shoutcastbuffer), NULL)==FALSE)
    g_printerr("GTK2: can't parse xiph xml file\n");
  if(shoutcastbuffer) g_free(shoutcastbuffer);
  shoutcastbuffer = 0;
  return(TRUE);
}

static void		cleaning_icecastlist(void)
{
}

static xml_icecastState ice_parser_state = XML_ICE_UNKNOW;

static void ice_parser_start_element(GMarkupParseContext *context,
				     const gchar         *element_name,
				     const gchar        **attribute_names,
				     const gchar        **attribute_values,
				     gpointer             user_data,
				     GError             **error)
{
  //g_print("start: %s\n", element_name);

  switch(ice_parser_state){
  case XML_ICE_UNKNOW:
    {
      if((!g_ascii_strcasecmp(element_name, "directory"))){
	ice_parser_state = XML_ICE_DIRECTORY;
      }
      break;
    }

  case XML_ICE_DIRECTORY:
    {
      if((!g_ascii_strcasecmp(element_name, "entry"))){
	ice_parser_state = XML_ICE_ENTRY;
	gtk_list_store_append(GTK_LIST_STORE(icecast_model), &iter);
      }
      break;
    }

  case XML_ICE_ENTRY:
    {
      if((!g_ascii_strcasecmp(element_name, "server_name"))){
	ice_parser_state = XML_ICE_SERVERNAME;
      }else if((!g_ascii_strcasecmp(element_name, "listen_url"))){
	ice_parser_state = XML_ICE_LISTENURL;
      }else if((!g_ascii_strcasecmp(element_name, "server_type"))){
	ice_parser_state = XML_ICE_SERVERTYPE;
      }else if((!g_ascii_strcasecmp(element_name, "bitrate"))){
	ice_parser_state = XML_ICE_BITRATE;
      }else if((!g_ascii_strcasecmp(element_name, "channels"))){
	ice_parser_state = XML_ICE_CHANNELS;
      }else if((!g_ascii_strcasecmp(element_name, "samplerate"))){
	ice_parser_state = XML_ICE_SAMPLERATE;
      }else if((!g_ascii_strcasecmp(element_name, "genre"))){
	ice_parser_state = XML_ICE_GENRE;
      }else if((!g_ascii_strcasecmp(element_name, "current_song"))){
	ice_parser_state = XML_ICE_CURRENTSONG;
      }
      break;
    }

  case XML_ICE_SERVERNAME:
  case XML_ICE_LISTENURL:
  case XML_ICE_SERVERTYPE:
  case XML_ICE_BITRATE:
  case XML_ICE_CHANNELS:
  case XML_ICE_SAMPLERATE:
  case XML_ICE_GENRE:
  case XML_ICE_CURRENTSONG:
    {
      break;
    }

  default:
    {
      g_printerr("GTK2: You should not be here XML is bad..\n");
      ice_parser_state = XML_ICE_UNKNOW;
      break;
    }
  }
}

static void ice_parser_end_element(GMarkupParseContext *context,
				   const gchar         *element_name,
				   gpointer             user_data,
				   GError             **error)
{
  switch(ice_parser_state){
  case XML_ICE_UNKNOW:
    {
      break;
    }
  case XML_ICE_DIRECTORY:
    {
      if((!g_ascii_strcasecmp(element_name, "directory"))){
	ice_parser_state = XML_ICE_UNKNOW;
      }
      break;
    }
  case XML_ICE_ENTRY:
    {
      if((!g_ascii_strcasecmp(element_name, "entry"))){
	ice_parser_state = XML_ICE_DIRECTORY;
      }
      break;
    }
  case XML_ICE_SERVERNAME:
    {
      if((!g_ascii_strcasecmp(element_name, "server_name"))){
	ice_parser_state = XML_ICE_ENTRY;
      }
      break;
    }
  case XML_ICE_LISTENURL:
    {
      if((!g_ascii_strcasecmp(element_name, "listen_url"))){
	ice_parser_state = XML_ICE_ENTRY;
      }
      break;
    }
  case XML_ICE_SERVERTYPE:
    {
      if((!g_ascii_strcasecmp(element_name, "server_type"))){
	ice_parser_state = XML_ICE_ENTRY;
      }
      break;
    }
  case XML_ICE_BITRATE:
    {
      if((!g_ascii_strcasecmp(element_name, "bitrate"))){
	ice_parser_state = XML_ICE_ENTRY;
      }
      break;
    }
  case XML_ICE_CHANNELS:
    {
      if((!g_ascii_strcasecmp(element_name, "channels"))){
	ice_parser_state = XML_ICE_ENTRY;
      }
      break;
    }
  case XML_ICE_SAMPLERATE:
    {
      if((!g_ascii_strcasecmp(element_name, "samplerate"))){
	ice_parser_state = XML_ICE_ENTRY;
      }
      break;
    }
  case XML_ICE_GENRE:
    {
      if((!g_ascii_strcasecmp(element_name, "genre"))){
	ice_parser_state = XML_ICE_ENTRY;
      }
      break;
    }
  case XML_ICE_CURRENTSONG:
    {
      if((!g_ascii_strcasecmp(element_name, "current_song"))){
	ice_parser_state = XML_ICE_ENTRY;
      }
      break;
    }
  default:
    {
      g_printerr("GTK2: You should not be here XML is bad..\n");
      ice_parser_state = XML_ICE_UNKNOW;
      break;
    }
  }
}

static void ice_parser_text_element(GMarkupParseContext *context,
                                    const gchar         *text,
                                    gsize                text_len,  
                                    gpointer             user_data,
                                    GError             **error)
{
  switch(ice_parser_state){
  case XML_ICE_UNKNOW:
    {
      break;
    }
  case XML_ICE_DIRECTORY:
  case XML_ICE_ENTRY:
    {
      break;
    }
  case XML_ICE_SERVERNAME:
    {
      gtk_list_store_set(GTK_LIST_STORE(icecast_model), &iter,
			 ICE_SERVERNAME, text, -1);
      break;
    }
  case XML_ICE_LISTENURL:
    {
      gtk_list_store_set(GTK_LIST_STORE(icecast_model), &iter,
			 ICE_LISTENURL, text, -1);
    }
  case XML_ICE_SERVERTYPE:
    {
      gtk_list_store_set(GTK_LIST_STORE(icecast_model), &iter,
			 ICE_STREAMTYPE, text, -1);
      break;
    }
  case XML_ICE_BITRATE:
    {
      gtk_list_store_set(GTK_LIST_STORE(icecast_model), &iter,
			 ICE_BITRATE, text, -1);
      break;
    }
  case XML_ICE_CHANNELS:
    {
      gtk_list_store_set(GTK_LIST_STORE(icecast_model), &iter,
			 ICE_CHANNELS, text, -1);
      break;
    }
  case XML_ICE_SAMPLERATE:
    {
      break;
    }
  case XML_ICE_GENRE:
    {
      gtk_list_store_set(GTK_LIST_STORE(icecast_model), &iter,
			 ICE_GENRE, text, -1);
      break;
    }
  case XML_ICE_CURRENTSONG:
    {
      break;
    }
  default:
    {
      g_printerr("GTK2: You should not be here, bad XML\n");
      ice_parser_state = XML_ICE_UNKNOW;
    }
  }
}

static gboolean	cb_icecast_treeview_selected(GtkWidget* widget,
					     GdkEventButton* event,
					     gpointer data)
{
  if(event->button == 1 && event->type == GDK_2BUTTON_PRESS){
    GtkTreeSelection*	selection;
    GtkTreeIter		iter;
    GtkTreeModel*	model;

    //g_print("double clicked !!!\n");
    selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(icecast_treeview));
    if(gtk_tree_selection_get_selected(selection, &model, &iter)){
      gchar*		url;
      lamipPlaylist*	playlist;
      lamipSonginfo*	info;
      GtkTreeSelection*	selection2;

      selection2 =
	gtk_tree_view_get_selection(GTK_TREE_VIEW(playlist_treeview));
      gtk_tree_model_get(model, &iter, ICE_LISTENURL, &url, -1);
#ifdef DEBUG
        g_printerr("You selected %s\n", url);
#endif
        lamip_stop();
        playlist_clean();
        playlist = lamip_playlist_new();
        playlist_add_item(playlist); // add to list of playlist
        info = lamip_songinfo_new();
        lamip_songinfo_read(url, info);
        lamip_playlist_append(playlist, info);
        lamip_playlist_set(playlist, 0, 0);
        g_free(url);
        // select the correct playlist
        playlist_selected = playlist;
        gtk_tree_selection_select_path(selection2,
                                       gtk_tree_path_new_from_string("0"));
        lamip_play();
    }
    return(FALSE);
  }else if(event->button == 1 && event->type == GDK_BUTTON_PRESS){
    //g_print("clicked !!!\n");
    return(FALSE);
  }
  return(TRUE);
}

/*****************************************************************************
 * PEERCAST part
 *
*/

static GtkTreeModel*	peercast_create_model(void)
{
  GtkListStore*	store;

  store = gtk_list_store_new(N_PEER_COLUMNS,
			     G_TYPE_STRING, // id of the server
			     G_TYPE_STRING, // title
			     G_TYPE_STRING, // genre
			     G_TYPE_STRING, // type
			     G_TYPE_STRING, // bitrate
			     G_TYPE_STRING,  // age
			     G_TYPE_STRING);// comment 
  
  return(GTK_TREE_MODEL(store));
}

// we use the same buffer text of shoutcast

static GMarkupParser peer_parserFuncs =
  {
    peer_parser_start_element,
    peer_parser_end_element,
    peer_parser_text_element,
    NULL,
    NULL
  };

static gboolean		creating_peercastlist(void)
{
  gint			ret = 0;
  GMarkupParseContext*	context;
  gsize read, write;
  gchar* utf8buffer;

#ifdef DEBUG
  g_printerr("GTK2: creating peercast list\n");
#endif
  gtk_list_store_clear(GTK_LIST_STORE(peercast_model));
  shoutcastbuffer = g_strdup("");
  curl = curl_easy_init();
  if(!curl) return(FALSE);

  curl_easy_setopt(curl, CURLOPT_URL, PEERCAST_LIST);
  curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, _curl_write2buffer);
  curl_easy_setopt(curl, CURLOPT_VERBOSE, 0);
  curl_easy_setopt(curl, CURLOPT_NOPROGRESS, 1);
  curl_easy_setopt(curl, CURLOPT_CONNECTTIMEOUT, 15);
  curl_easy_setopt(curl, CURLOPT_LOW_SPEED_LIMIT, 0);
  curl_easy_setopt(curl, CURLOPT_LOW_SPEED_TIME, 15);
  curl_easy_setopt(curl, CURLOPT_USERAGENT, "lamip/" VERSION);
  curl_easy_setopt(curl, CURLOPT_HTTPGET, 1);
  ret = curl_easy_perform(curl);
  if(ret != CURLE_OK){
    g_printerr("GTK2: Can't download the peercast streams list");
    return(FALSE);
  }
  /* the peercastlist string is full now, let's add content
   * to the treeview
   */
#ifdef DEBUG
  g_printerr("%s", shoutcastbuffer);
#endif

  utf8buffer = g_convert_with_fallback(shoutcastbuffer, strlen(shoutcastbuffer), "UTF-8", "ISO-8859-15", "fred", &read, &write, 0);
  if(!utf8buffer)
    g_print("GTK2: utf8 conversion failed\n");

  context = g_markup_parse_context_new(&peer_parserFuncs, 0, NULL, NULL);
  if(g_markup_parse_context_parse(context, utf8buffer, strlen(utf8buffer), NULL)==FALSE)
    g_printerr("GTK2: can't parse yellowpage_peercast xml file\n");
  if(shoutcastbuffer) g_free(shoutcastbuffer);
  if(utf8buffer) g_free(utf8buffer);
  shoutcastbuffer = 0;
  return(TRUE);
}

static void		cleaning_peercastlist(void)
{
}

static xml_peercastState peer_parser_state = XML_PEER_UNKNOW;

static void peer_parser_start_element(GMarkupParseContext *context,
				      const gchar         *element_name,
				      const gchar        **attribute_names,
				      const gchar        **attribute_values,
				      gpointer             user_data,
				      GError             **error)
{
  //g_print("start: %d - %s", peer_parser_state, element_name);

  switch(peer_parser_state){
  case XML_PEER_UNKNOW:
    {
      if((!g_ascii_strcasecmp(element_name, "peercast"))){
	peer_parser_state = XML_PEER_PEERCAST;
      }
      break;
    }

  case XML_PEER_PEERCAST:
    {
      if((!g_ascii_strcasecmp(element_name, "servent"))){
	int i;
	peer_parser_state = XML_PEER_SERVENT;
	for(i=0;attribute_names && attribute_names[i]!=NULL;i+=1){
	  if(!g_ascii_strcasecmp(attribute_names[i], "uptime")){
	  }
	}
      }else if((!g_ascii_strcasecmp(element_name, "bandwidth"))){
	int i;
	peer_parser_state = XML_PEER_BANDWIDTH;
     
	for(i=0;attribute_names && attribute_names[i]!=NULL;i+=1){
	  if(!g_ascii_strcasecmp(attribute_names[i], "out")){
	  }else if(!g_ascii_strcasecmp(attribute_names[i], "in")){
	  }
	}
      }else if((!g_ascii_strcasecmp(element_name, "connections"))){
	int i;
	peer_parser_state = XML_PEER_CONNECTIONS;
      
	for(i=0;attribute_names && attribute_names[i]!=NULL;i+=1){
	  if(!g_ascii_strcasecmp(attribute_names[i], "total")){
	  }else if(!g_ascii_strcasecmp(attribute_names[i], "relay")){
	  }else if(!g_ascii_strcasecmp(attribute_names[i], "direct")){
	  }
	}
      }else if((!g_ascii_strcasecmp(element_name, "channels_relayed"))){
	peer_parser_state = XML_PEER_CHANNELS_RELAYED;
      }else if((!g_ascii_strcasecmp(element_name, "channels_found"))){
	int i;

	peer_parser_state = XML_PEER_CHANNELS_FOUND;
	for(i=0;attribute_names && attribute_names[i]!=NULL;i+=1){
	  if(!g_ascii_strcasecmp(attribute_names[i], "total")){
	  }
	}
      }
      break;
    }
  case XML_PEER_SERVENT:
    {
      break;
    }
  case XML_PEER_BANDWIDTH:
    {
      break;
    }
  case XML_PEER_CONNECTIONS:
    {
      break;
    }
  case XML_PEER_CHANNELS_RELAYED:
    {
      int i;
      
      for(i=0;attribute_names && attribute_names[i]!=NULL;i+=1){
	if(!g_ascii_strcasecmp(attribute_names[i], "total")){
	}
      }
      break;
    }
  case XML_PEER_CHANNELS_FOUND:
    {
      int i;

      if((!g_ascii_strcasecmp(element_name, "channel"))){
	peer_parser_state = XML_PEER_CHANNEL;
	gtk_list_store_append(GTK_LIST_STORE(peercast_model), &iter);

	for(i=0;attribute_names && attribute_names[i]!=NULL;i+=1){
	  if(!g_ascii_strcasecmp(attribute_names[i], "name")){
	    gtk_list_store_set(GTK_LIST_STORE(peercast_model), &iter,
			       PEER_CHANNEL_NAME, attribute_values[i], -1);
	  }else if(!g_ascii_strcasecmp(attribute_names[i], "id")){
	    gtk_list_store_set(GTK_LIST_STORE(peercast_model), &iter,
			       PEER_ID, attribute_values[i], -1);
	  }else if(!g_ascii_strcasecmp(attribute_names[i], "bitrate")){
	    gtk_list_store_set(GTK_LIST_STORE(peercast_model), &iter,
			       PEER_BITRATE, attribute_values[i], -1);
	  }else if(!g_ascii_strcasecmp(attribute_names[i], "type")){
	    gtk_list_store_set(GTK_LIST_STORE(peercast_model), &iter,
			       PEER_TYPE, attribute_values[i], -1);
	  }else if(!g_ascii_strcasecmp(attribute_names[i], "genre")){
	    gtk_list_store_set(GTK_LIST_STORE(peercast_model), &iter,
			       PEER_GENRE, attribute_values[i], -1);
	  }else if(!g_ascii_strcasecmp(attribute_names[i], "desc")){
	  }else if(!g_ascii_strcasecmp(attribute_names[i], "url")){
	  }else if(!g_ascii_strcasecmp(attribute_names[i], "uptime")){
	  }else if(!g_ascii_strcasecmp(attribute_names[i], "comment")){
	    gtk_list_store_set(GTK_LIST_STORE(peercast_model), &iter,
			       PEER_COMMENT, attribute_values[i], -1);
	  }else if(!g_ascii_strcasecmp(attribute_names[i], "skips")){
	  }else if(!g_ascii_strcasecmp(attribute_names[i], "age")){
	    gtk_list_store_set(GTK_LIST_STORE(peercast_model), &iter,
			       PEER_AGE, attribute_values[i], -1);
	  }
	}
      }
      break;
    }
  case XML_PEER_CHANNEL:
    {
      if((!g_ascii_strcasecmp(element_name, "hits"))){
	int i;

	peer_parser_state = XML_PEER_HITS;
	for(i=0;attribute_names && attribute_names[i]!=NULL;i+=1){
	  if(!g_ascii_strcasecmp(attribute_names[i], "listeners")){
	  }else if(!g_ascii_strcasecmp(attribute_names[i], "hosts")){
	  }else if(!g_ascii_strcasecmp(attribute_names[i], "firewalled")){
	  }else if(!g_ascii_strcasecmp(attribute_names[i], "closest")){
	  }else if(!g_ascii_strcasecmp(attribute_names[i], "furthest")){
	  }else if(!g_ascii_strcasecmp(attribute_names[i], "newest")){
	  }
	}
      }else if((!g_ascii_strcasecmp(element_name, "track"))){
	int i;
	peer_parser_state = XML_PEER_TRACK;

	for(i=0;attribute_names && attribute_names[i]!=NULL;i+=1){
	  if(!g_ascii_strcasecmp(attribute_names[i], "title")){
	  }else if(!g_ascii_strcasecmp(attribute_names[i], "artist")){
	  }else if(!g_ascii_strcasecmp(attribute_names[i], "album")){
	  }else if(!g_ascii_strcasecmp(attribute_names[i], "genre")){
	  }else if(!g_ascii_strcasecmp(attribute_names[i], "contact")){
	  }
	}
      }
      break;
    }
  case XML_PEER_HITS:
    {
      if((!g_ascii_strcasecmp(element_name, "host"))){
	int i;

	peer_parser_state = XML_PEER_HITS;
	for(i=0;attribute_names && attribute_names[i]!=NULL;i+=1){
	  if(!g_ascii_strcasecmp(attribute_names[i], "ip")){
	  }else if(!g_ascii_strcasecmp(attribute_names[i], "hops")){
	  }else if(!g_ascii_strcasecmp(attribute_names[i], "listeners")){
	  }else if(!g_ascii_strcasecmp(attribute_names[i], "relays")){
	  }else if(!g_ascii_strcasecmp(attribute_names[i], "uptime")){
	  }else if(!g_ascii_strcasecmp(attribute_names[i], "push")){
	  }else if(!g_ascii_strcasecmp(attribute_names[i], "relay")){
	  }else if(!g_ascii_strcasecmp(attribute_names[i], "direct")){
	  }else if(!g_ascii_strcasecmp(attribute_names[i], "cin")){
	  }else if(!g_ascii_strcasecmp(attribute_names[i], "stable")){
	  }else if(!g_ascii_strcasecmp(attribute_names[i], "agent")){
	  }else if(!g_ascii_strcasecmp(attribute_names[i], "version")){
	  }else if(!g_ascii_strcasecmp(attribute_names[i], "uptime")){
	  }else if(!g_ascii_strcasecmp(attribute_names[i], "tracker")){
	  }	  
	}
      }
      break;
    }
  case XML_PEER_HOST:
    {
      break;
    }
  case XML_PEER_TRACK:
    {
      break;
    }
  default:
    {
      g_printerr("GTK2: You should not be here XML is bad..\n");
      peer_parser_state = XML_PEER_UNKNOW;
      break;
    }
  }
}

static void peer_parser_end_element(GMarkupParseContext *context,
				    const gchar         *element_name,
				    gpointer             user_data,
				    GError             **error)
{
  //g_print("end %d - %s\n", peer_parser_state, element_name);

  switch(peer_parser_state){
  case XML_PEER_UNKNOW:
    {
      break;
    }
  case XML_PEER_PEERCAST:
    {
      if((!g_ascii_strcasecmp(element_name, "peercast"))){
	peer_parser_state = XML_ICE_UNKNOW;
      }
      break;
    }
  case XML_PEER_SERVENT:
    {
      if((!g_ascii_strcasecmp(element_name, "servent"))){
	peer_parser_state = XML_PEER_PEERCAST;
      }
      break;
    }
  case XML_PEER_BANDWIDTH:
    {
      if((!g_ascii_strcasecmp(element_name, "bandwidth"))){
	peer_parser_state = XML_PEER_PEERCAST;
      }
      break;
    }
  case XML_PEER_CONNECTIONS:
    {
      if((!g_ascii_strcasecmp(element_name, "connections"))){
	peer_parser_state = XML_PEER_PEERCAST;
      }
      break;
    }
  case XML_PEER_CHANNELS_RELAYED:
    {
      if((!g_ascii_strcasecmp(element_name, "channels_relayed"))){
	peer_parser_state = XML_PEER_PEERCAST;
      }
      break;
    }
  case XML_PEER_CHANNELS_FOUND:
    {
      if((!g_ascii_strcasecmp(element_name, "channels_found"))){
	peer_parser_state = XML_PEER_PEERCAST;
      }
      break;
    }
  case XML_PEER_CHANNEL:
    {
      if((!g_ascii_strcasecmp(element_name, "channel"))){
	peer_parser_state = XML_PEER_CHANNELS_FOUND;
      }
      break;
    }
  case XML_PEER_HITS:
    {
      if((!g_ascii_strcasecmp(element_name, "hits"))){
	peer_parser_state = XML_PEER_CHANNEL;
      }
      break;
    }
  case XML_PEER_HOST:
    {
      if((!g_ascii_strcasecmp(element_name, "host"))){
	peer_parser_state = XML_PEER_HITS;
      }
      break;
    }
  case XML_PEER_TRACK:
    {
      if((!g_ascii_strcasecmp(element_name, "track"))){
	peer_parser_state = XML_PEER_CHANNEL;
      }
      break;
    }
  default:
    {
      g_printerr("GTK2: You should not be here XML is bad..\n");
      peer_parser_state = XML_ICE_UNKNOW;
      break;
    }
  }
}

static void peer_parser_text_element(GMarkupParseContext *context,
				     const gchar         *text,
				     gsize                text_len,  
				     gpointer             user_data,
				     GError             **error)
{
  switch(peer_parser_state){
  case XML_PEER_PEERCAST:
  case XML_PEER_SERVENT:
  case XML_PEER_BANDWIDTH:
  case XML_PEER_CONNECTIONS:
  case XML_PEER_CHANNELS_RELAYED:
  case XML_PEER_CHANNELS_FOUND:
  case XML_PEER_CHANNEL:
  case XML_PEER_HITS:
  case XML_PEER_HOST:
  case XML_PEER_TRACK:
  case XML_ICE_UNKNOW:
    {
      break;
    }
  default:
    {
      g_printerr("GTK2: You should not be here, bad XML\n");
      peer_parser_state = XML_PEER_UNKNOW;
    }
  }
}

static gboolean	cb_peercast_treeview_selected(GtkWidget* widget,
					      GdkEventButton* event,
					      gpointer data)
{
  if(event->button == 1 && event->type == GDK_2BUTTON_PRESS){
    GtkTreeSelection*	selection;
    GtkTreeIter		iter;
    GtkTreeModel*	model;

    //g_print("double clicked !!!\n");
    selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(peercast_treeview));
    if(gtk_tree_selection_get_selected(selection, &model, &iter)){
      gchar*		url;
      gchar*		tmp;
      lamipPlaylist*	playlist;
      lamipSonginfo*	info;
      GtkTreeSelection*	selection2;

      if(!peercast_hostname)
	return(FALSE);
      selection2 =
	gtk_tree_view_get_selection(GTK_TREE_VIEW(playlist_treeview));
      gtk_tree_model_get(model, &iter, PEER_ID, &url, -1);
	
      tmp = g_strconcat("http://", peercast_hostname, ":7144/stream/", url, NULL);
#ifdef DEBUG
        g_printerr("You selected peercast '%s' url\n", tmp);
#endif
	g_free(url);
	url = tmp;
        lamip_stop();
        playlist_clean();
        playlist = lamip_playlist_new();
        playlist_add_item(playlist); // add to list of playlist
        info = lamip_songinfo_new();
        lamip_songinfo_read(url, info);
        lamip_playlist_append(playlist, info);
        lamip_playlist_set(playlist, 0, 0);
        g_free(url);
        // select the correct playlist
        playlist_selected = playlist;
        gtk_tree_selection_select_path(selection2,
                                       gtk_tree_path_new_from_string("0"));
        lamip_play();
    }
    return(FALSE);
  }else if(event->button == 1 && event->type == GDK_BUTTON_PRESS){
    //g_print("clicked !!!\n");
    return(FALSE);
  }
  return(TRUE);
}


#endif /* WITH_SHOUTCAST */
